@@ -13,6 +13,7 @@
 #include "Plugin.hpp"
 #include "ConfigVar.hpp"
 #include "HTTPHeader.hpp"
+#include "ListContainer.hpp"
 
 // DEFINES
 
@@ -31,8 +32,18 @@
 // allow access to http[s] based auth system, but with no persitance
 #define DGAUTH_OK_NOPERSIST 9
 
+// auth plugin found a partial or incomplet answer (Eg NTLM): just break ident loop for this request  
+#define DGAUTH_NOIDENTPART 5
+
 // any < 0 return code signifies error
 
+// auth_rec structure for use by storyboarding
+struct auth_rec {
+    String user_name;
+    bool is_authed;
+    int filter_group;
+};
+
 // DECLARATIONS
 
 class AuthPlugin : public Plugin
@@ -49,7 +60,7 @@ class AuthPlugin : public Plugin
     // REDIRECT - redirect user to URL in string
     // NOMATCH - did not find the necessary info in the request (query remaining plugins)
     // any < 0 - error
-    virtual int identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string) = 0;
+    virtual int identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string, bool &is_real_user) = 0;
 
     // determine what filter group the given username is in
     // queries the standard filtergroupslist
@@ -58,13 +69,15 @@ class AuthPlugin : public Plugin
     // NOMATCH - did not find a group for this user (query remaining plugins)
     // NOUSER - did not find a group for this user (do not query remaining plugins)
     // any < 0 - error
-    virtual int determineGroup(std::string &user, int &fg);
+    virtual int determineGroup(std::string &user, int &fg,ListContainer &uglc);
 
     // is this a connection-based auth type, i.e. assume all subsequent requests on the pconn are from the same user?
     bool is_connection_based;
 
     // does this auth type rely on queries from the parent proxy (e.g. NTLM, basic auth)?
     bool needs_proxy_query;
+    bool client_ip_based;
+
     String getPluginName();
     virtual bool isTransparent()
     {
