@@ -32,17 +32,18 @@
 // GLOBALS
 
 extern OptionContainer o;
+extern thread_local std::string thread_id;
 
 // IMPLEMENTATION
 
 DataBuffer::DataBuffer()
-    : data(new char[1]), buffer_length(0), compresseddata(NULL), compressed_buffer_length(0), tempfilesize(0), dontsendbody(false), tempfilefd(-1), dm_plugin(NULL), timeout(20), bytesalreadysent(0), preservetemp(false), stimeout(20000)
+    : data(new char[1]), buffer_length(0), compresseddata(NULL), compressed_buffer_length(0), tempfilesize(0), dontsendbody(false), tempfilefd(-1), dm_plugin(NULL), timeout(20000), stimeout(20), bytesalreadysent(0), preservetemp(false), icap(false)
 {
     data[0] = '\0';
 }
 
 DataBuffer::DataBuffer(const void *indata, off_t length)
-    : data(new char[length]), buffer_length(length), compresseddata(NULL), compressed_buffer_length(0), tempfilesize(0), dontsendbody(false), tempfilefd(-1), timeout(20), bytesalreadysent(0), preservetemp(false)
+    : data(new char[length]), buffer_length(length), compresseddata(NULL), compressed_buffer_length(0), tempfilesize(0), dontsendbody(false), tempfilefd(-1), timeout(20000), stimeout(20), bytesalreadysent(0), preservetemp(false), icap(false)
 {
     memcpy(data, indata, length);
 }
@@ -73,6 +74,8 @@ void DataBuffer::reset()
     dontsendbody = false;
     preservetemp = false;
     decompress = "";
+    chunked = false;
+    icap = false;
 }
 
 // delete the memory block when the class is destroyed
@@ -111,27 +114,27 @@ void DataBuffer::swapbacktocompressed()
 // the buffer gets filled thus reducing memcpy()ing and new()ing
 int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int sockettimeout)
 {
-    int pos = 0;
-    int rc;
-    while (pos < size) {
-        rc = sock->readFromSocket(&buffer[pos], size - pos, 0, sockettimeout);
-        if (rc < 1) {
-            // none recieved or an error
-            if (pos > 0) {
-                return pos; // some was recieved previous into buffer
+        int pos = 0;
+        int rc;
+        while (pos < size) {
+            rc = sock->readFromSocket(&buffer[pos], size - pos, 0, sockettimeout);
+            if (rc < 1) {
+                // none recieved or an error
+                if (pos > 0) {
+                    return pos; // some was recieved previous into buffer
+                }
+                return rc; // just return with the return code
             }
-            return rc; // just return with the return code
+            pos += rc;
         }
-        pos += rc;
-    }
-    return size; // full buffer
+        return size; // full buffer
 }
 
 // a much more efficient reader that does not assume the contents of
 // the buffer gets filled thus reducing memcpy()ing and new()ing.
 // in addition to the actual socket timeout, used for each individual read, this version
 // incorporates a "global" timeout within which all reads must complete.
-int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int sockettimeout, int timeout)
+int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int sockettimeout, int stimeout)
 {
 
     int pos = 0;
@@ -140,7 +143,11 @@ int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int s
     struct timeval nowadays;
     gettimeofday(&starttime, NULL);
     while (pos < size) {
-        rc = sock->readFromSocket(&buffer[pos], size - pos, 0, sockettimeout, false);
+        if (chunked) {
+            rc = sock->readChunk(&buffer[pos], size - pos,sockettimeout );
+        } else {
+            rc = sock->readFromSocket(&buffer[pos], size - pos, 0, sockettimeout, false);
+        }
         if (rc < 1) {
             // none recieved or an error
             if (pos > 0) {
@@ -150,9 +157,9 @@ int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int s
         }
         pos += rc;
         gettimeofday(&nowadays, NULL);
-        if (nowadays.tv_sec - starttime.tv_sec > timeout) {
+        if (nowadays.tv_sec - starttime.tv_sec > stimeout) {
 #ifdef DGDEBUG
-            std::cout << "buffered socket read more than timeout" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+            std::cerr << thread_id << "buffered socket read more than timeout" << std::endl;
 #endif
             return pos; // just return how much got so far then
         }
@@ -174,9 +181,9 @@ int DataBuffer::getTempFileFD()
     umask(0007); // only allow access to e2g user and group
     if ((tempfilefd = mkstemp(tempfilepatharray)) < 0) {
 #ifdef DGDEBUG
-        std::cout << "error creating temp " << tempfilepath << ": " << strerror(errno) << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+        std::cerr << thread_id << "error creating temp " << tempfilepath << ": " << strerror(errno) << std::endl;
 #endif
-        syslog(LOG_ERR, "Could not create temp file to store download for scanning: %s", strerror(errno));
+        syslog(LOG_ERR, "%sCould not create temp file to store download for scanning: %s", thread_id.c_str(), strerror(errno));
         tempfilefd = -1;
         tempfilepath = "";
     } else {
@@ -210,7 +217,7 @@ bool DataBuffer::in(Socket *sock, Socket *peersock, HTTPHeader *requestheader, H
     for (std::deque<Plugin *>::iterator i = o.dmplugins_begin; i != o.dmplugins_end; i++) {
         if ((i + 1) == o.dmplugins_end) {
 #ifdef DGDEBUG
-            std::cout << "Got to final download manager so defaulting to always match." << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+            std::cerr << thread_id << "Got to final download manager so defaulting to always match." << std::endl;
 #endif
             dm_plugin = (DMPlugin *)(*i);
             rc = dm_plugin->in(this, sock, peersock, requestheader, docheader, runav, headersent, &toobig);
@@ -218,7 +225,7 @@ bool DataBuffer::in(Socket *sock, Socket *peersock, HTTPHeader *requestheader, H
         } else {
             if (((DMPlugin *)(*i))->willHandle(requestheader, docheader)) {
 #ifdef DGDEBUG
-                std::cout << "Matching download manager number: " << j << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+                std::cerr << thread_id << "Matching download manager number: " << j << std::endl;
 #endif
                 dm_plugin = (DMPlugin *)(*i);
                 rc = dm_plugin->in(this, sock, peersock, requestheader, docheader, runav, headersent, &toobig);
@@ -239,16 +246,16 @@ bool DataBuffer::out(Socket *sock) //throw(std::exception)
 {
     if (dontsendbody) {
 #ifdef DGDEBUG
-        std::cout << "dontsendbody true; not sending" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+        std::cerr << thread_id << "dontsendbody true; not sending" << std::endl;
 #endif
         return true;
     }
-    if (!(*sock).breadyForOutput(stimeout)) return false; // exceptions on timeout or error
+    if (!(*sock).breadyForOutput(timeout)) return false; // exceptions on timeout or error
 
     if (tempfilefd > -1) {
 // must have been too big for ram so stream from disk in blocks
 #ifdef DGDEBUG
-        std::cout << "Sending " << tempfilesize - bytesalreadysent << " bytes from temp file (" << bytesalreadysent << " already sent)" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+        std::cerr << thread_id << "Sending " << tempfilesize - bytesalreadysent << " bytes from temp file (" << bytesalreadysent << " already sent)" << std::endl;
 #endif
         off_t sent = bytesalreadysent;
         int rc;
@@ -256,59 +263,98 @@ bool DataBuffer::out(Socket *sock) //throw(std::exception)
         if (lseek(tempfilefd, bytesalreadysent, SEEK_SET) < 0)
             return false;
 //            throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
+         int block_len;
+        if(chunked)
+            block_len = 4048;
+        else
+            block_len = buffer_length;
 
         while (sent < tempfilesize) {
-            rc = readEINTR(tempfilefd, data, buffer_length);
+            rc = readEINTR(tempfilefd, data, block_len);
 #ifdef DGDEBUG
-            std::cout << "reading temp file rc:" << rc << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+            std::cerr << thread_id << "reading temp file rc:" << rc << std::endl;
 #endif
             if (rc < 0) {
 #ifdef DGDEBUG
-                std::cout << "error reading temp file so throwing exception" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+                std::cerr << thread_id << "error reading temp file so throwing exception" << std::endl;
 #endif
                 return false;
     //            throw std::exception();
             }
             if (rc == 0) {
 #ifdef DGDEBUG
-                std::cout << "got zero bytes reading temp file" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+                std::cerr << thread_id << "got zero bytes reading temp file" << std::endl;
 #endif
                 break; // should never happen
             }
             // as it's cached to disk the buffer must be reasonably big
-            if (!sock->writeToSocket(data, rc, 0, stimeout)) {
-                return false;
-//                throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
+            if(chunked) {
+                if (!sock->writeChunk(data, rc, timeout))
+                    return false;
+            } else {
+                if (!sock->writeToSocket(data, rc, 0, timeout)) {
+                    return false;
+                }
             }
             sent += rc;
 #ifdef DGDEBUG
-            std::cout << "total sent from temp:" << sent << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+            std::cerr << thread_id << "total sent from temp:" << sent << std::endl;
 #endif
         }
+        if (chunked && got_all) {
+            String n;
+            if (!sock->writeChunkTrailer(n))
+                return false;
+        }
         close(tempfilefd);
         tempfilefd = -1;
         tempfilesize = 0;
         unlink(tempfilepath.toCharArray());
     } else {
+        off_t sent = bytesalreadysent;
 #ifdef DGDEBUG
-        std::cout << "Sending " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length << " in buffer; " << bytesalreadysent << " already sent)" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+        std::cerr << thread_id << "Sending " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length << " in buffer; " << bytesalreadysent << " already sent)" << std::endl;
 #endif
         // it's in RAM, so just send it, no streaming from disk
+        int block_len;
+        if(chunked)
+            block_len = 4048;
+        else
+            block_len = buffer_length;
+
         if (buffer_length != 0) {
-            if (!sock->writeToSocket(data + bytesalreadysent, buffer_length - bytesalreadysent, 0, stimeout))
-                return false;
-          //      throw std::exception();
+            while (sent < buffer_length) {
+                if( block_len > (buffer_length - sent))
+                    block_len = (buffer_length - sent);
+                if (chunked) {
+                    if (!sock->writeChunk(data + sent, block_len, timeout))
+                        return false;
+
+                } else {
+                    if (!sock->writeToSocket(data + sent, buffer_length - sent, 0, timeout))
+                        return false;
+                }
+                sent += block_len;
+            }
+            if (chunked && got_all) {
+                String n;
+                if (!sock->writeChunkTrailer(n))
+                    return false;
+            }
         } else {
-            if (!sock->writeToSocket("\r\n\r\n", 4, 0, stimeout))
-                return false;
-           //     throw std::exception();
+            if(chunked) {
+                if (!sock->writeChunk(data + bytesalreadysent, 0, timeout))
+                    return false;
+            } else {
+                if (!sock->writeToSocket("\r\n\r\n", 4, 0, timeout))
+                    return false;
+            }
         }
 #ifdef DGDEBUG
-        std::cout << "Sent " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length  << ") Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+        std::cerr << thread_id << "Sent " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length  << std::endl;
 #endif
-     return true;
     }
-   return true;
+    return true;
 }
 
 // zlib decompression
@@ -318,7 +364,7 @@ void DataBuffer::zlibinflate(bool header)
         return; // it can't possibly be zlib'd
     }
 #ifdef DGDEBUG
-    std::cout << "compressed size:" << buffer_length << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+    std::cerr << thread_id << "compressed size:" << buffer_length << std::endl;
 #endif
 
 #if ZLIB_VERNUM < 0x1210
@@ -362,13 +408,13 @@ void DataBuffer::zlibinflate(bool header)
     if (err != Z_OK) { // was a problem so just return
         delete[] block; // don't forget to free claimed memory
 #ifdef DGDEBUG
-        std::cout << "bad init inflate: " << err << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+        std::cerr << thread_id << "bad init inflate: " << err << std::endl;
 #endif
         return;
     }
     while (true) {
 #ifdef DGDEBUG
-        std::cout << "inflate loop" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+        std::cerr << thread_id << "inflate loop" << std::endl;
 #endif
         err = inflate(&d_stream, Z_SYNC_FLUSH);
         bytesgot = d_stream.total_out;
@@ -377,7 +423,7 @@ void DataBuffer::zlibinflate(bool header)
             if (err != Z_OK) {
                 delete[] block;
 #ifdef DGDEBUG
-                std::cout << "bad inflateEnd: " << d_stream.msg << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+                std::cerr << thread_id << "bad inflateEnd: " << d_stream.msg << std::endl;
 #endif
                 return;
             }
@@ -386,12 +432,12 @@ void DataBuffer::zlibinflate(bool header)
         if (err != Z_OK) { // was a problem so just return
             delete[] block; // don't forget to free claimed memory
 #ifdef DGDEBUG
-            std::cout << "bad inflate: " << d_stream.msg << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+            std::cerr << thread_id << "bad inflate: " << d_stream.msg << std::endl;
 #endif
             err = inflateEnd(&d_stream);
             if (err != Z_OK) {
 #ifdef DGDEBUG
-                std::cout << "bad inflateEnd: " << d_stream.msg << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+                std::cerr << thread_id << "bad inflateEnd: " << d_stream.msg << std::endl;
 #endif
             }
             return;
@@ -399,12 +445,12 @@ void DataBuffer::zlibinflate(bool header)
         if (bytesgot > o.max_content_filter_size) {
             delete[] block; // don't forget to free claimed memory
 #ifdef DGDEBUG
-            std::cout << "inflated file larger than maxcontentfiltersize, not inflating further" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+            std::cerr << thread_id << "inflated file larger than maxcontentfiltersize, not inflating further" << std::endl;
 #endif
             err = inflateEnd(&d_stream);
             if (err != Z_OK) {
 #ifdef DGDEBUG
-                std::cout << "bad inflateEnd: " << d_stream.msg << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+                std::cerr << thread_id << "bad inflateEnd: " << d_stream.msg << std::endl;
 #endif
             }
             return;
@@ -426,7 +472,7 @@ void DataBuffer::zlibinflate(bool header)
     compressed_buffer_length = buffer_length;
     buffer_length = bytesgot;
 #ifdef DGDEBUG
-    std::cout << "decompressed size: " << buffer_length << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+    std::cerr << thread_id << "decompressed size: " << buffer_length << std::endl;
 #endif
     data = new char[bytesgot + 1];
     data[bytesgot] = '\0';
@@ -443,7 +489,7 @@ bool DataBuffer::contentRegExp(FOptionContainer* &foc)
 {
 
 #ifdef DGDEBUG
-    std::cout << "Starting content reg exp replace" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+    std::cerr << thread_id << "Starting content reg exp replace" << std::endl;
 #endif
     bool contentmodified = false;
     unsigned int i;
@@ -523,7 +569,7 @@ bool DataBuffer::contentRegExp(FOptionContainer* &foc)
             matches = m;
 
 #ifdef DGDEBUG
-            std::cout << "content matches:" << matches << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
+            std::cerr << thread_id << "content matches:" << matches << std::endl;
 #endif
             // replace top-level matches using filled-out replacement strings
             newreplacement *newrep;
@@ -553,3 +599,14 @@ bool DataBuffer::contentRegExp(FOptionContainer* &foc)
     }
     return contentmodified;
 }
+
+void DataBuffer::setChunked(bool ch = true) {
+    chunked = ch;
+    return;
+}
+
+void DataBuffer::setICAP(bool ch = true) {
+    chunked = ch;
+    icap = ch;
+    return;
+}
