@@ -26,6 +26,7 @@
 
 extern bool is_daemonised;
 extern OptionContainer o;
+extern thread_local std::string thread_id;
 
 // IMPLEMENTATION
 
@@ -51,13 +52,14 @@ bool HTMLTemplate::readTemplateFile(const char *filename, const char *placeholde
     // compile regexp for matching supported placeholders
     // allow optional custom placeholder string
     re.comp(placeholders ? placeholders : "-URL-|-REASONGIVEN-|-REASONLOGGED-|-USER-|-IP-|-HOST-|-FILTERGROUP-|-RAWFILTERGROUP-|-BYPASS-|-CATEGORIES-|-SHORTURL-|-SERVERIP-");
+    RegResult Rre;
     unsigned int offset;
     String result;
     String line;
     std::ifstream templatefile(filename, std::ios::in); // e2guardian.conf
     if (!templatefile.good()) {
         if (!is_daemonised) {
-            std::cerr << "error reading: " << filename << std::endl;
+            std::cerr << thread_id << "error reading: " << filename << std::endl;
         }
         syslog(LOG_ERR, "%s", "error reading HTML template file.");
         return false;
@@ -66,11 +68,11 @@ bool HTMLTemplate::readTemplateFile(const char *filename, const char *placeholde
         std::getline(templatefile, linebuffer);
         line = linebuffer.c_str();
         // look for placeholders
-        re.match(line.toCharArray());
-        while (re.numberOfMatches() > 0) {
+        re.match(line.toCharArray(),Rre);
+        while (Rre.numberOfMatches() > 0) {
             // whenever we find one, push the text before it onto the list, then the placeholder, then the text after it
-            offset = re.offset(0);
-            result = re.result(0).c_str();
+            offset = Rre.offset(0);
+            result = Rre.result(0).c_str();
             if (offset > 0) {
                 push(line.subString(0, offset));
                 push(result);
@@ -79,7 +81,7 @@ bool HTMLTemplate::readTemplateFile(const char *filename, const char *placeholde
                 push(result);
                 line = line.subString(result.length(), line.length() - result.length());
             }
-            re.match(line.toCharArray());
+            re.match(line.toCharArray(),Rre);
         }
         // if any text remains, or we didn't find a placeholder, push the remainder of the line
         if (line.length() > 0) {
@@ -99,13 +101,10 @@ void makeURLSafe(String &url)
     url.replaceall(">", "%3E");
 }
 
-// fill in placeholders with the given information and send the resulting page to the client
-// only useful if you used the default set of placeholders
-void HTMLTemplate::display(Socket *s, String *url, std::string &reason, std::string &logreason, std::string &categories,
-    std::string *user, std::string *ip, std::string *host, int filtergroup, String &hashed)
-{
+void HTMLTemplate::display_hb(String &ebody, String *url, std::string &reason, std::string &logreason, std::string &categories,
+                           std::string *user, std::string *ip, std::string *host, int filtergroup, String grpname, String &hashed , String &localip) {
 #ifdef DGDEBUG
-    std::cout << "Displaying TEMPLATE" << std::endl;
+    std::cerr << thread_id << "Displaying TEMPLATE" << std::endl;
 #endif
     String line;
     bool newline;
@@ -134,7 +133,7 @@ void HTMLTemplate::display(Socket *s, String *url, std::string &reason, std::str
                 line += "...";
             }
         } else if (line == "-SERVERIP-") {
-            line = s->getLocalIP();
+            line = localip;
         } else if (line == "-REASONGIVEN-") {
             String safereason = reason;
             makeURLSafe(safereason);
@@ -149,18 +148,20 @@ void HTMLTemplate::display(Socket *s, String *url, std::string &reason, std::str
             line = *ip;
         } else if (line == "-HOST-") {
             if (host == NULL) {
-#ifdef DGDEBUG
-                std::cout << "-HOST- placeholder encountered but hostname currently unknown; lookup forced." << std::endl;
-#endif
-                std::deque<String> *names = ipToHostname(ip->c_str());
-                if (names->size() > 0) {
-                    *host = names->front();
-                }
-                delete names;
+//#ifdef DGDEBUG
+                //std::cerr << thread_id << "-HOST- placeholder encountered but hostname currently unknown; lookup forced." << std::endl;
+//#endif
+                //std::deque<String> *names = ipToHostname(ip->c_str());
+                //if (names->size() > 0) {
+                    //*host = names->front();
+                //}
+                //delete names;
+                line = "";
+            } else {
+                line = *host;
             }
-            line = (host ? *host : "");
         } else if (line == "-FILTERGROUP-") {
-            line = o.fg[filtergroup]->name;
+            line = grpname;
         } else if (line == "-RAWFILTERGROUP-") {
             line = String(filtergroup + 1);
         } else if (line == "-CATEGORIES-") {
@@ -192,12 +193,24 @@ void HTMLTemplate::display(Socket *s, String *url, std::string &reason, std::str
             }
         }
         if (line.length() > 0) {
-            s->writeString(line.toCharArray());
+            ebody += line;
         }
         if (newline) {
-            s->writeString("\n");
+            ebody += "\n";
         }
     }
-    s->writeString(html[sz].toCharArray());
-    s->writeString("\n");
+    ebody += html[sz].toCharArray();
+    ebody += "\n";
+}
+
+// fill in placeholders with the given information and send the resulting page to the client
+// only useful if you used the default set of placeholders
+    void HTMLTemplate::display(Socket *s, String *url, std::string &reason, std::string &logreason, std::string &categories,
+                               std::string *user, std::string *ip, std::string *host, int filtergroup, String grpname, String &hashed)
+{
+    String ebody;
+    String localip = s->getLocalIP();
+    display_hb(ebody, url, reason, logreason, categories,
+                               user, ip, host, filtergroup, grpname, hashed, localip);
+    s->writeString(ebody.toCharArray());
 }
