@@ -22,6 +22,7 @@
 //#include "DataBuffer.hpp"
 #include "Socket.hpp"
 #include "RegExp.hpp"
+#include "ListMeta.hpp"
 #include "FOptionContainer.hpp"
 
 // DECLARATIONS
@@ -32,7 +33,13 @@ class HTTPHeader
     std::deque<String> header;
     //DataBuffer postdata;
     unsigned int port;
-    bool is_response;
+    bool is_response = false;
+    bool icap;
+    String redirect;
+    String useragent;
+    String contenttype;
+    String contentencoding;
+    String transferencoding;
 
     // reset header object for future use
     void reset();
@@ -40,7 +47,11 @@ class HTTPHeader
     // network communication funcs
 
     void setTimeout(int t);
-    bool in(Socket *sock, bool allowpersistent = false, bool honour_reloadconfig = false);
+    bool in(Socket *sock, bool allowpersistent = false );
+
+    void setClientIP(String &ip);
+
+    String stringHeader();  // output header as a String (used by ICAP)
 
     // send headers out over the given socket
     // "reconnect" flag gives permission to reconnect to the socket on write error
@@ -58,7 +69,10 @@ class HTTPHeader
 
     // request type: GET, HEAD, POST etc.
     String requestType();
+    String requesttype;
     int returnCode();
+    int returncode;
+
     // get content length - returns -1 if undetermined
     off_t contentLength();
     String getContentType();
@@ -77,19 +91,24 @@ class HTTPHeader
     bool isRedirection();
     // see if content-type is something other than "identity"
     bool isCompressed();
+    bool addHeader(String & xheader);
     //bool isHeaderAdded(int filtergroup);
     bool isHeaderAdded(FOptionContainer* &foc);
     bool addheaderchecked;
     bool isheaderadded;
     String *plogheadervalue;
     String *pheaderident;
+    String *ptransfercoding;
+    String *ptransferencoding;
     std::string getAuthHeader();
     // see if search usl and set searchwords
     bool isSearch(FOptionContainer* &foc);
     String searchwords();
     String searchterms();
     bool searchchecked;
+    bool chunked;
     String contentEncoding();
+    String transferEncoding();
     // grab the contents of Proxy-Authorization header
     // returns base64-decoding of the chunk of data after the auth type string
     std::string getAuthData();
@@ -114,6 +133,9 @@ class HTTPHeader
     // is called.
     void setPostData(const char *data, size_t len);
 
+    void setDirect();
+
+
     // detailed value/type checks
 
     bool malformedURL(const String &url);
@@ -130,12 +152,8 @@ class HTTPHeader
     // strip content-encoding, and simultaneously set content-length to newlen
     void removeEncoding(int newlen);
     void setContentLength(int newlen);
-    // regexp search and replace
-    bool urlRegExp(FOptionContainer* &foc);
-    bool sslsiteRegExp(FOptionContainer* &foc);
-    bool urlRedirectRegExp(FOptionContainer* &foc);
+
     bool DenySSL(FOptionContainer* &foc);
-    bool headerRegExp(FOptionContainer* &foc);
     // make a connection persistent - or not
     void makePersistent(bool persist = true);
     // make the request look as if its coming from the origin server
@@ -143,7 +161,6 @@ class HTTPHeader
     // modifies the URL in all relevant header lines after a regexp search and replace
     // setURL Code originally from from Ton Gorter 2004
     void setURL(String &url);
-
     // do URL decoding (%xx) on string
     // decode everything, or just numbers, letters and -
     static String decode(const String &s, bool decodeAll = false);
@@ -151,19 +168,15 @@ class HTTPHeader
     // Bypass URL & Cookie funcs
 
     // is this a temporary filter bypass URL?
-    int isBypassURL(String *url, const char *magic, const char *clientip, bool *isvirusbypass);
+    int isBypassURL(String url, const char *magic, const char *clientip, bool *isvirusbypass);
     // is this a scan bypass URL? (download previously scanned file)
-    bool isScanBypassURL(String *url, const char *magic, const char *clientip);
-    bool isMITMAcceptURL(String *url, const char *magic, const char *clientip);
-    // is this a temporary filter bypass cookie?
+    bool isScanBypassURL(String url, const char *magic, const char *clientip);
     bool isBypassCookie(String url, const char *magic, const char *clientip);
-    bool isMITMAcceptCookie(String url, const char *magic, const char *clientip);
-    // chop GBYPASS/GSPYBASS off URLs (must know it's there to begin with)
     void chopBypass(String url, bool infectionbypass);
     void chopScanBypass(String url);
-    void chopMITMAccept(String url);
     // add cookie to outgoing headers with given name & value
     void setCookie(const char *cookie, const char *domain, const char *value);
+    bool isProxyRequest;
 
     // encode url
     String URLEncode();
@@ -172,25 +185,35 @@ class HTTPHeader
     String getReferer();
 
     HTTPHeader()
-        : port(0), timeout(120000), contentlength(0), postdata(NULL), dirty(true), is_response(false)
+    //    : port(0), timeout(120000), contentlength(0), postdata(NULL), dirty(true), is_response(false)
+        : port(0), timeout(120000), contentlength(0), postdata(NULL), dirty(true)
     {
         reset();
     };
     HTTPHeader(int type)
-            : port(0), timeout(120000), contentlength(0), postdata(NULL), dirty(true), is_response(false)
+    //        : port(0), timeout(120000), contentlength(0), postdata(NULL), dirty(true), is_response(false)
+    : port(0), timeout(120000), contentlength(0), postdata(NULL), dirty(true)
     {
         reset();
-        if (type == __HEADER_RESPONSE)
-            is_response = true;
+        setType(type);
     };
+
     ~HTTPHeader()
     {
         delete postdata;
     };
 
-    private:
-    // timeout for socket operations
-    int timeout;
+    void setType(int type) {
+        if (type == __HEADER_RESPONSE)
+            is_response = true;
+        else
+            is_response = false;
+    };
+
+       private:
+        // timeout for socket operations
+        int timeout;
+
 
     // header index pointers
     String *phost;
@@ -210,9 +233,10 @@ class HTTPHeader
     // cached result of getUrl()
     std::string cachedurl;
     // used to record if it is a header within a MITM
-    bool mitm;
+    bool mitm = false;
+    // is direct rather than via proxy
+    bool isdirect = false;
 
-    String redirect;
     String searchwds;
     //std::string searchwds;
     //std::string searchtms;
@@ -231,6 +255,8 @@ class HTTPHeader
 
     bool dirty;
 
+    std::string s_clientip;
+
     // check & fix headers from servers that don't obey standards
     void checkheader(bool allowpersistent);
 
