@@ -28,6 +28,7 @@
 
 extern bool is_daemonised;
 extern OptionContainer o;
+extern thread_local std::string thread_id;
 
 // DEFINES
 
@@ -42,7 +43,7 @@ extern OptionContainer o;
 // Constructor - set default values
 ListContainer::ListContainer()
     : refcount(0), parent(false), filedate(0), used(false), bannedpfiledate(0), exceptionpfiledate(0), weightedpfiledate(0), blanketblock(false), blanket_ip_block(false), blanketsslblock(false), blanketssl_ip_block(false), sourceisexception(false), sourcestartswith(false), sourcefilters(0), data(NULL), current_graphdata_size(0), realgraphdata(NULL), maxchildnodes(0), graphitems(0), data_length(0), data_memory(0), items(0), isSW(false), issorted(false), graphused(false), force_quick_search(false),
-    /*sthour(0), stmin(0), endhour(0), endmin(0),*/ istimelimited(false)
+    /*sthour(0), stmin(0), endhour(0), endmin(0),*/ istimelimited(false), is_iplist(false)
 {
 }
 
@@ -67,7 +68,7 @@ void ListContainer::reset()
     if (refcount > 0) {
         --refcount;
 #ifdef DGDEBUG
-        std::cout << "de-reffing " << sourcefile << " due to manual list reset, refcount: " << refcount << std::endl;
+//        std::cerr << thread_id << "de-reffing " << sourcefile << " due to manual list reset, refcount: " << refcount << std::endl;
 #endif
         for (size_t i = 0; i < morelists.size(); ++i)
             o.lm.deRefList(morelists[i]);
@@ -110,6 +111,11 @@ void ListContainer::reset()
     bannedpfiledate = 0;
     exceptionpfiledate = 0;
     weightedpfiledate = 0;
+    if(is_iplist) {
+        iplist.clear();
+        iprangelist.clear();
+        ipsubnetlist.clear();
+    }
 }
 
 // for item lists - during a config reload, can we simply retain the already loaded list?
@@ -141,7 +147,7 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception, int c
         len = getFileLength(filename);
     } catch (std::runtime_error &e) {
         if (!is_daemonised) {
-            std::cerr << "Error reading file (does it exist?) " << filename << ": " << e.what() << std::endl;
+            std::cerr << thread_id << "Error reading file (does it exist?) " << filename << ": " << e.what() << std::endl;
         }
         syslog(LOG_ERR, "Error reading file (does it exist?) %s: %s", filename, e.what());
         return false;
@@ -155,7 +161,7 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception, int c
     std::ifstream listfile(filename, std::ios::in); // open the file for reading
     if (!listfile.good()) {
         if (!is_daemonised) {
-            std::cerr << "Error opening file (does it exist?): " << filename << std::endl;
+            std::cerr << thread_id << "Error opening file (does it exist?): " << filename << std::endl;
         }
         syslog(LOG_ERR, "Error opening file (does it exist?): %s", filename);
         return false;
@@ -194,15 +200,15 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception, int c
                 // returning index) if it is not.
                 catindex = getCategoryIndex(&lcat);
 #ifdef DGDEBUG
-                std::cout << "List category: " << lcat << std::endl;
-                std::cout << "Category list index: " << catindex << std::endl;
+                std::cerr << thread_id << "List category: " << lcat << std::endl;
+                std::cerr << thread_id << "Category list index: " << catindex << std::endl;
 #endif
             }
             // phrase lists can also be marked as not to be case-converted,
             // to aid support for exotic character encodings
             else if (line.startsWith("#noconvert")) {
 #ifdef DGDEBUG
-                std::cout << "List flagged as not to be case-converted" << std::endl;
+                std::cerr << thread_id << "List flagged as not to be case-converted" << std::endl;
 #endif
                 caseinsensitive = false;
             }
@@ -215,7 +221,7 @@ bool ListContainer::readPhraseList(const char *filename, bool isexception, int c
                 timelimits.push_back(tl);
                 timeindex = timelimits.size() - 1;
 #ifdef DGDEBUG
-                std::cout << "Found time limit on phrase list. Now have " << timelimits.size() << " limits on this list (including parents)." << std::endl;
+                std::cerr << thread_id << "Found time limit on phrase list. Now have " << timelimits.size() << " limits on this list (including parents)." << std::endl;
 #endif
                 continue;
             }
@@ -284,7 +290,7 @@ void ListContainer::readPhraseListHelper2(String phrase, int type, int weighting
 
     if (phrase.length() > 127) {
         if (!is_daemonised) {
-            std::cerr << "Phrase length too long, truncating: " << phrase << std::endl;
+            std::cerr << thread_id << "Phrase length too long, truncating: " << phrase << std::endl;
         }
         syslog(LOG_ERR, "Phrase length too long, truncating: %s", phrase.toCharArray());
         phrase = phrase.subString(0, 127);
@@ -297,7 +303,7 @@ void ListContainer::readPhraseListHelper2(String phrase, int type, int weighting
     if (type < 10) {
         if (!addToItemListPhrase(phrase.toCharArray(), phrase.length(), type, weighting, false, catindex, timeindex)) {
             if (!is_daemonised) {
-                std::cerr << "Duplicate phrase, dropping: " << phrase << std::endl;
+                std::cerr << thread_id << "Duplicate phrase, dropping: " << phrase << std::endl;
             }
             syslog(LOG_ERR, "Duplicate phrase, dropping: %s", phrase.toCharArray());
         }
@@ -332,14 +338,29 @@ bool ListContainer::addToItemListPhrase(const char *s, size_t len, int type, int
     return true;
 }
 
-bool ListContainer::ifsreadItemList(std::ifstream *input, int len, bool checkendstring, const char *endstring, bool do_includes, bool startswith, int filters)
+bool ListContainer::ifsreadItemList(std::istream *input, int len, bool checkendstring, const char *endstring, bool do_includes, bool startswith, int filters)
 {
+    int mem_used = 2;
     RegExp re;
     re.comp("^.*\\:[0-9]+\\/.*");
     RegResult Rre;
+    if(is_iplist) {
+#ifdef HAVE_PCRE
+        matchIP.comp("^\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}$");
+    matchSubnet.comp("^\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}/\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}$");
+    matchCIDR.comp("^\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}/\\d{1,2}$");
+    matchRange.comp("^\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}-\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}$");
+#else
+        matchIP.comp("^[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}$");
+        matchSubnet.comp("^[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}/[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}$");
+        matchCIDR.comp("^[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}/[0-9]{1,2}$");
+        matchRange.comp("^[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}-[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}$");
+#endif
+    }
+
 #ifdef DGDEBUG
     if (filters != 32)
-        std::cout << "Converting to lowercase" << std::endl;
+        std::cerr << thread_id << "Converting to lowercase" << std::endl;
 #endif
     increaseMemoryBy(len + 2); // Allocate some memory to hold file
     String temp, inc, hostname, url;
@@ -381,7 +402,7 @@ bool ListContainer::ifsreadItemList(std::ifstream *input, int len, bool checkend
             } else if (temp.startsWith("#listcategory:")) {
                 category = temp.after("\"").before("\"");
 #ifdef DGDEBUG
-                std::cout << "found item list category: " << category << std::endl;
+                std::cerr << thread_id << "found item list category: " << category << std::endl;
 #endif
                 continue;
             } else if (checkendstring && temp.startsWith(endstring)) {
@@ -390,20 +411,6 @@ bool ListContainer::ifsreadItemList(std::ifstream *input, int len, bool checkend
             continue; // it's a comment
         }
 
-        // blanket block flags
-        if (temp.startsWith("**s")) {
-            blanketsslblock = true;
-            continue;
-        } else if (temp.startsWith("**ips")) {
-            blanketssl_ip_block = true;
-            continue;
-        } else if (temp.startsWith("**")) {
-            blanketblock = true;
-            continue;
-        } else if (temp.startsWith("*ip")) {
-            blanket_ip_block = true;
-            continue;
-        }
 
         // Strip off comments that don't necessarily start at the beginning of a line
         // - but not regular expression comments
@@ -453,12 +460,17 @@ bool ListContainer::ifsreadItemList(std::ifstream *input, int len, bool checkend
         if (filters != 32) {
             temp.toLower(); // tidy up - but don't make regex lists lowercase!
         }
-        if (temp.length() > 0)
-            addToItemList(temp.toCharArray(), temp.length()); // add to unsorted list
+        if (temp.length() > 0) {
+            mem_used += temp.length() + 1;
+            if (is_iplist)
+                addToIPList(temp);
+            else {
+                if (mem_used > data_memory)
+                    increaseMemoryBy(2048);
+                addToItemList(temp.toCharArray(), temp.length()); // add to unsorted list
+            }
+        }
     }
-#ifdef DGDEBUG
-    std::cout << "Blanket flags set:" << blanketblock << ":" << blanket_ip_block << ":" << blanketsslblock << ":" << blanketssl_ip_block << std::endl;
-#endif
     return true; // sucessful read
 }
 
@@ -469,7 +481,7 @@ bool ListContainer::ifsReadSortItemList(std::ifstream *input, bool checkendstrin
         len = getFileLength(filename);
     } catch (std::runtime_error &e) {
         if (!is_daemonised) {
-            std::cerr << "Error reading file " << filename << ": " << e.what() << std::endl;
+            std::cerr << thread_id << "Error reading file " << filename << ": " << e.what() << std::endl;
         }
         syslog(LOG_ERR, "Error reading file %s: %s", filename, e.what());
         return false;
@@ -484,18 +496,22 @@ bool ListContainer::ifsReadSortItemList(std::ifstream *input, bool checkendstrin
 }
 
 // for item lists - read item list from file. checkme - what is startswith? is it used? what is filters?
-bool ListContainer::readItemList(const char *filename, bool startswith, int filters)
+bool ListContainer::readItemList(const char *filename, bool startswith, int filters, bool isip)
 {
     ++refcount;
     sourcefile = filename;
     sourcestartswith = startswith;
     sourcefilters = filters;
+
+    if(isip) is_iplist = true;
+    else is_iplist = false;
+
+    if (sourcefile.startsWithLower("memory:"))
+        return readStdinItemList(startswith, filters);
     std::string linebuffer;
 #ifdef DGDEBUG
-    std::cout << "Read filename: " << filename << std::endl;
+    std::cerr << thread_id << filename << std::endl;
 #endif
-    // see if cached .process file is available and up to date,
-    // or prefercachedlists has been turned on (SG)
     struct stat s;
     filedate = getFileDate(filename);
     size_t len = 0;
@@ -503,7 +519,7 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
         len = getFileLength(filename);
     } catch (std::runtime_error &e) {
         if (!is_daemonised) {
-            std::cerr << "Error reading file " << filename << ": " << e.what() << std::endl;
+            std::cerr << thread_id << "Error reading file " << filename << ": " << e.what() << std::endl;
         }
         syslog(LOG_ERR, "Error reading file %s: %s", filename, e.what());
         return false;
@@ -515,7 +531,7 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
     std::ifstream listfile(filename, std::ios::in);
     if (!listfile.good()) {
         if (!is_daemonised) {
-            std::cerr << "Error opening: " << filename << std::endl;
+            std::cerr << thread_id << "Error opening: " << filename << std::endl;
         }
         syslog(LOG_ERR, "Error opening file: %s", filename);
         return false;
@@ -523,7 +539,7 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
     if (!ifsreadItemList(&listfile, len, false, NULL, true, startswith, filters)) {
         listfile.close();
         if (!is_daemonised) {
-            std::cerr << "Error reading: " << filename << std::endl;
+            std::cerr << thread_id << "Error reading: " << filename << std::endl;
         }
         syslog(LOG_ERR, "Error reading file: %s", filename);
         return false;
@@ -533,100 +549,44 @@ bool ListContainer::readItemList(const char *filename, bool startswith, int filt
 }
 
 // for stdin item lists - read item list from stdin
-bool ListContainer::readStdinItemList(bool startswith, int filters, const char *startstr)
-{
+bool ListContainer::readStdinItemList(bool startswith, int filters) {
 #ifdef DGDEBUG
     if (filters != 32)
-        std::cout << "Converting to lowercase" << std::endl;
+        std::cerr << thread_id << "Converting to lowercase" << std::endl;
 #endif
-    int mem_used = 2; // keep 2 bytes spare??
-    bool skip = true;
-    sourcestartswith = startswith;
-    sourcefilters = filters;
     std::string linebuffer;
     RegExp re;
     re.comp("^.*\\:[0-9]+\\/.*");
     RegResult Rre;
-    size_t len = 0;
+    size_t len = 2046;
     increaseMemoryBy(2048); // Allocate some memory to hold list
     if (!std::cin.good()) {
         if (!is_daemonised) {
-            std::cerr << "Error reading stdin: " << std::endl;
+            std::cerr << thread_id << "Error reading stdin: " << std::endl;
         }
         syslog(LOG_ERR, "Error reading stdin");
         return false;
     }
-    String temp, inc, hostname, url;
-    while (!std::cin.eof()) {
-        getline(std::cin, linebuffer);
-        if (linebuffer.length() < 2)
-            continue; // its jibberish
 
-        temp = linebuffer.c_str();
-
-        if (linebuffer[0] == '#') {
-            if (skip && temp.startsWith(startstr)) {
-                skip = false;
-                continue;
-            }
-            if (temp.startsWith("#ENDLIST")) {
-                break; // end of list
-            } else {
-                continue; // it's a comment
-            }
+    if (!ifsreadItemList(&std::cin, len, true, "#ENDLIST", false, startswith, filters)) {
+        if (!is_daemonised) {
+            std::cerr << thread_id << "Error reading stdin: " << std::endl;
         }
-
-        if (skip)
-            continue;
-
-        // Strip off comments that don't necessarily start at the beginning of a line
-        std::string::size_type commentstart = 1;
-        while ((commentstart = temp.find_first_of('#', commentstart)) != std::string::npos) {
-            if (temp[commentstart - 1] == ' ') {
-                temp = temp.substr(0, commentstart);
-                break;
-            }
-            ++commentstart;
-        }
-
-        temp.removeWhiteSpace(); // tidy up and make it handle CRLF files
-        if (temp.endsWith("/")) {
-            temp.chop(); // tidy up
-        }
-        if (temp.startsWith("ftp://")) {
-            temp = temp.after("ftp://"); // tidy up
-        }
-        if (filters == 1) { // remove port addresses
-            if (temp.before("/").contains(":")) { // quicker than full regexp
-                if (re.match(temp.toCharArray(),Rre)) {
-                    hostname = temp.before(":");
-                    url = temp.after("/");
-                    temp = hostname + "/" + url;
-                }
-            }
-        }
-        if (filters != 32) {
-            temp.toLower(); // tidy up - but don't make regex lists lowercase!
-        }
-        if (temp.length() > 0)
-            mem_used += temp.length() + 1;
-        if (mem_used > data_memory) {
-            increaseMemoryBy(2048);
-        }
-        addToItemList(temp.toCharArray(), temp.length()); // add to unsorted list
-    }
-    //listfile.close();
-    return true; // sucessful read
+        syslog(LOG_ERR, "Error reading stdin");
+        return false;
+    } else
+        return true;
 }
 
+
 // for item lists - read nested item lists
 bool ListContainer::readAnotherItemList(const char *filename, bool startswith, int filters)
 {
 
-    int result = o.lm.newItemList(filename, startswith, filters, false);
+    int result = o.lm.newItemList(filename, startswith, filters, false, is_iplist);
     if (result < 0) {
         if (!is_daemonised) {
-            std::cerr << "Error opening file: " << filename << std::endl;
+            std::cerr << thread_id << "Error opening file: " << filename << std::endl;
         }
         syslog(LOG_ERR, "Error opening file: %s", filename);
         return false;
@@ -689,10 +649,16 @@ bool ListContainer::inListStartsWith(const char *string, String &lastcategory)
 }
 
 // find pointer to the part of the data array containing this string
-char *ListContainer::findInList(const char *string, String &lastcategory)
+const char *ListContainer::findInList(const char *string, String &lastcategory)
 {
     if (isNow()) {
-        if (items > 0) {
+        if (is_iplist) {
+            if(inIPList(string) != NULL)
+            {
+                lastcategory = category;
+                return "";    //TODO return IP/IPblock/IPrange matched
+            }
+        } else if (items > 0) {
             int r;
             if (isSW) {
                 r = search(&ListContainer::greaterThanSWF, 0, items - 1, string);
@@ -704,7 +670,7 @@ char *ListContainer::findInList(const char *string, String &lastcategory)
                 return (data + list[r]);
             }
         }
-        char *rc;
+        const char *rc;
         for (unsigned int i = 0; i < morelists.size(); i++) {
             rc = (*o.lm.l[morelists[i]]).findInList(string, lastcategory);
             if (rc != NULL) {
@@ -867,6 +833,10 @@ void ListContainer::doSort(const bool startsWith)
 { // sort by ending of line
     for (size_t i = 0; i < morelists.size(); i++)
         (*o.lm.l[morelists[i]]).doSort(startsWith);
+    if (is_iplist) {
+        std::sort(iplist.begin(), iplist.end());
+        return;
+    }
     if (items < 2 || issorted)
         return;
     if (startsWith) {
@@ -935,7 +905,7 @@ bool ListContainer::makeGraph(bool fqs)
     graphused = true;
 
 #ifdef DGDEBUG
-    std::cout << "Bytes needed for phrase tree in worst-case scenario: " << (sizeof(int) * ((GRAPHENTRYSIZE * data_length) + ROOTOFFSET))
+    std::cerr << thread_id << "Bytes needed for phrase tree in worst-case scenario: " << (sizeof(int) * ((GRAPHENTRYSIZE * data_length) + ROOTOFFSET))
               << ", starting off with allocation of " << (sizeof(int) * ((GRAPHENTRYSIZE * ((data_length / 3) + 1)) + ROOTOFFSET)) << std::endl;
     prolificroot = false;
     secondmaxchildnodes = 0;
@@ -961,10 +931,10 @@ bool ListContainer::makeGraph(bool fqs)
     }
 
 #ifdef DGDEBUG
-    std::cout << "Bytes actually needed for phrase tree: " << (sizeof(int) * ((GRAPHENTRYSIZE * graphitems) + ROOTOFFSET)) << std::endl;
-    std::cout << "Most prolific node has " << maxchildnodes << " children" << std::endl;
-    std::cout << "It " << (prolificroot ? "is" : "is not") << " the root node" << std::endl;
-    std::cout << "Second most prolific node has " << secondmaxchildnodes << " children" << std::endl;
+    std::cerr << thread_id << "Bytes actually needed for phrase tree: " << (sizeof(int) * ((GRAPHENTRYSIZE * graphitems) + ROOTOFFSET)) << std::endl;
+    std::cerr << thread_id << "Most prolific node has " << maxchildnodes << " children" << std::endl;
+    std::cerr << thread_id << "It " << (prolificroot ? "is" : "is not") << " the root node" << std::endl;
+    std::cerr << thread_id << "Second most prolific node has " << secondmaxchildnodes << " children" << std::endl;
 #endif
 
     realgraphdata = (int *)realloc(realgraphdata, sizeof(int) * ((GRAPHENTRYSIZE * graphitems) + ROOTOFFSET));
@@ -1201,11 +1171,11 @@ void ListContainer::graphSearch(std::map<std::string, std::pair<unsigned int, in
 
     if (force_quick_search || graphitems == 0) {
 #ifdef DGDEBUG
-        std::cout << "Map (quicksearch) start" << std::endl;
+        std::cerr << thread_id << "Map (quicksearch) start" << std::endl;
         for (std::map<std::string, std::pair<unsigned int, int> >::iterator i = result.begin(); i != result.end(); i++) {
-            std::cout << "Map: " << i->first << " " << i->second.second << std::endl;
+            std::cerr << thread_id << "Map: " << i->first << " " << i->second.second << std::endl;
         }
-        std::cout << "Map (quicksearch) end" << std::endl;
+        std::cerr << thread_id << "Map (quicksearch) end" << std::endl;
 #endif
         return;
     }
@@ -1255,7 +1225,7 @@ void ListContainer::graphSearch(std::map<std::string, std::pair<unsigned int, in
                             existingitem->second.second++;
                         }
 #ifdef DGDEBUG
-                        std::cout << "Found this phrase: " << phrase << std::endl;
+                        std::cerr << thread_id << "Found this phrase: " << phrase << std::endl;
 #endif
                     }
                     // grab this node's number of children
@@ -1293,11 +1263,11 @@ void ListContainer::graphSearch(std::map<std::string, std::pair<unsigned int, in
         }
     }
 #ifdef DGDEBUG
-    std::cout << "Map start" << std::endl;
+    std::cerr << thread_id << "Map start" << std::endl;
     for (std::map<std::string, std::pair<unsigned int, int> >::iterator i = result.begin(); i != result.end(); i++) {
-        std::cout << "Map: " << i->first << " " << i->second.second << std::endl;
+        std::cerr << thread_id << "Map: " << i->first << " " << i->second.second << std::endl;
     }
-    std::cout << "Map end" << std::endl;
+    std::cerr << thread_id << "Map end" << std::endl;
 #endif
 }
 
@@ -1390,7 +1360,7 @@ void ListContainer::graphAdd(String s, const int inx, int item)
         if ((inx == 0) ? ((numlinks + 1) > MAXROOTLINKS) : ((numlinks + 1) > MAXLINKS)) {
             syslog(LOG_ERR, "Cannot load phraselists from this many languages/encodings simultaneously. (more than %d links from this node! [1])", (inx == 0) ? MAXROOTLINKS : MAXLINKS);
             if (!is_daemonised)
-                std::cout << "Cannot load phraselists from this many languages/encodings simultaneously. (more than " << ((inx == 0) ? MAXROOTLINKS : MAXLINKS) << " links from this node! [1])" << std::endl;
+                std::cerr << thread_id << "Cannot load phraselists from this many languages/encodings simultaneously. (more than " << ((inx == 0) ? MAXROOTLINKS : MAXLINKS) << " links from this node! [1])" << std::endl;
             exit(1);
         }
         if ((numlinks + 1) > maxchildnodes) {
@@ -1414,7 +1384,7 @@ void ListContainer::graphAdd(String s, const int inx, int item)
             if ((inx == 0) ? ((numlinks + 1) > MAXROOTLINKS) : ((numlinks + 1) > MAXLINKS)) {
                 syslog(LOG_ERR, "Cannot load phraselists from this many languages/encodings simultaneously. (more than %d links from this node! [2])", (inx == 0) ? MAXROOTLINKS : MAXLINKS);
                 if (!is_daemonised)
-                    std::cout << "Cannot load phraselists from this many languages/encodings simultaneously. (more than " << ((inx == 0) ? MAXROOTLINKS : MAXLINKS) << " links from this node! [2])" << std::endl;
+                    std::cerr << thread_id << "Cannot load phraselists from this many languages/encodings simultaneously. (more than " << ((inx == 0) ? MAXROOTLINKS : MAXLINKS) << " links from this node! [2])" << std::endl;
                 exit(1);
             }
             if ((numlinks + 1) > maxchildnodes) {
@@ -1456,98 +1426,6 @@ void ListContainer::graphAdd(String s, const int inx, int item)
     }
 }
 
-bool ListContainer::readProcessedItemList(const char *filename, bool startswith, int filters)
-{
-#ifdef DGDEBUG
-    std::cout << "reading processed file:" << filename << std::endl;
-#endif
-    size_t len = 0;
-    size_t slen;
-    try {
-        len = getFileLength(filename);
-    } catch (std::runtime_error &e) {
-        if (!is_daemonised) {
-            std::cerr << "Error reading file " << filename << ": " << e.what() << std::endl;
-        }
-        syslog(LOG_ERR, "Error reading file %s: %s", filename, e.what());
-        return false;
-    }
-    if (len < 5) {
-        if (!is_daemonised) {
-            std::cerr << "File too small (less than 5 bytes - is it corrupt?): " << filename << std::endl;
-        }
-        syslog(LOG_ERR, "%s", "File too small (less than 5 bytes - is it corrupt?):");
-        syslog(LOG_ERR, "%s", filename);
-        return false;
-    }
-    increaseMemoryBy(len + 2);
-    std::ifstream listfile(filename, std::ios::in);
-    if (!listfile.good()) {
-        if (!is_daemonised) {
-            std::cerr << "Error opening: " << filename << std::endl;
-        }
-        syslog(LOG_ERR, "Error opening: %s", filename);
-        return false;
-    }
-    std::string linebuffer;
-    String temp, inc;
-    while (!listfile.eof()) {
-        getline(listfile, linebuffer);
-        if (linebuffer[0] == '.') {
-            temp = linebuffer.c_str();
-            if (temp.startsWith(".Include<")) { // see if we have another list
-                inc = temp.after(".Include<").before(">"); // to include
-                if (!readAnotherItemList(inc.toCharArray(), startswith, filters)) { // read it
-                    listfile.close();
-                    return false;
-                }
-                continue;
-            }
-        } else if (linebuffer[0] == '#') {
-            temp = linebuffer.c_str();
-            if (temp.startsWith("#time: ")) { // see if we have a time tag
-                if (!readTimeTag(&temp, listtimelimit)) {
-                    return false;
-                }
-                continue;
-            } else if (temp.startsWith("#listcategory:")) { // see if we have a category
-                category = temp.after("\"").before("\"");
-#ifdef DGDEBUG
-                std::cout << "found item processed list category: " << category << std::endl;
-#endif
-                continue;
-            }
-            continue; // it's a comment man
-        }
-        // blanket block flags
-        if (temp.startsWith("**s")) {
-            blanketsslblock = true;
-            continue;
-        } else if (temp.startsWith("**ips")) {
-            blanketssl_ip_block = true;
-            continue;
-        } else if (temp.startsWith("**")) {
-            blanketblock = true;
-            continue;
-        } else if (temp.startsWith("*ip")) {
-            blanket_ip_block = true;
-            continue;
-        }
-        slen = linebuffer.length();
-        if (slen < 3)
-            continue;
-        list.push_back(data_length);
-        lengthlist.push_back(slen);
-        for (size_t i = 0; i < slen; i++) {
-            data[data_length + i] = linebuffer[i];
-        }
-        data[data_length + slen] = 0;
-        data_length += slen + 1;
-        items++;
-    }
-    listfile.close();
-    return true;
-}
 
 void ListContainer::addToItemList(const char *s, size_t len)
 {
@@ -1561,6 +1439,68 @@ void ListContainer::addToItemList(const char *s, size_t len)
     items++;
 }
 
+void ListContainer::addToIPList(String& line)
+{
+    RegResult Rre;
+
+    // store the IP address (numerically, not as a string) and filter group in either the IP list, subnet list or range list
+    if (matchIP.match(line.toCharArray(),Rre)) {
+        struct in_addr address;
+        if (inet_aton(line.toCharArray(), &address)) {
+            uint32_t addr = ntohl(address.s_addr);
+            iplist.push_back(addr);
+        }
+    } else if (matchSubnet.match(line.toCharArray(),Rre)) {
+        struct in_addr address;
+        struct in_addr addressmask;
+        String subnet(line.before("/"));
+        String mask(line.after("/"));
+        if (inet_aton(subnet.toCharArray(), &address) && inet_aton(mask.toCharArray(), &addressmask)) {
+            ipl_subnetstruct s;
+            uint32_t addr = ntohl(address.s_addr);
+            s.mask = ntohl(addressmask.s_addr);
+            // pre-mask the address for quick comparison
+            s.maskedaddr = addr & s.mask;
+            ipsubnetlist.push_back(s);
+        }
+    } else if (matchCIDR.match(line.toCharArray(),Rre)) {
+        struct in_addr address;
+        struct in_addr addressmask;
+        String subnet(line.before("/"));
+        String cidr(line.after("/"));
+        int m = cidr.toInteger();
+        int host_part = 32 - m;
+        if (host_part > -1) {
+            String mask = (0xFFFFFFFF << host_part);
+            if (inet_aton(subnet.toCharArray(), &address) && inet_aton(mask.toCharArray(), &addressmask)) {
+                ipl_subnetstruct s;
+                uint32_t addr = ntohl(address.s_addr);
+                s.mask = ntohl(addressmask.s_addr);
+                // pre-mask the address for quick comparison
+                s.maskedaddr = addr & s.mask;
+                ipsubnetlist.push_back(s);
+            }
+        }
+    } else if (matchRange.match(line.toCharArray(),Rre)) {
+        struct in_addr addressstart;
+        struct in_addr addressend;
+        String start(line.before("-"));
+        String end(line.after("-"));
+        if (inet_aton(start.toCharArray(), &addressstart) && inet_aton(end.toCharArray(), &addressend)) {
+            ipl_rangestruct r;
+            r.startaddr = ntohl(addressstart.s_addr);
+            r.endaddr = ntohl(addressend.s_addr);
+            iprangelist.push_back(r);
+        }
+#ifdef DGDEBUG
+        else
+        {
+        std::cerr << thread_id << "Not adding to any IP list" << line << std::endl;
+        }
+#endif
+    }
+}
+
 int ListContainer::search(int (ListContainer::*comparitor)(const char *a, const char *b), int a, int s, const char *p)
 {
     if (a > s)
@@ -1679,13 +1619,10 @@ time_t getFileDate(const char *filename)
 {
     struct stat status;
     int rc = stat(filename, &status);
-    if (strlen(filename) < 1)
-            return 0;
-	
     if (rc != 0) {
         if (errno == ENOENT) {
 #ifdef DGDEBUG
-            std::cout << "Cannot stat file m_time for " << filename << ". stat() returned errno ENOENT." << std::endl;
+            std::cerr << thread_id << "Cannot stat file m_time for " << filename << ". stat() returned errno ENOENT." << std::endl;
 #endif
             syslog(LOG_ERR, "Error reading %s. Check directory and file permissions. They should be 640 and 750: %s", filename, strerror(errno));
             return 0;
@@ -1693,14 +1630,14 @@ time_t getFileDate(const char *filename)
         // If there are permission problems, just reload the file (CN)
         if (errno == EACCES) {
 #ifdef DGDEBUG
-            std::cout << "Cannot stat file m_time for " << filename << ". stat() returned errno EACCES." << std::endl;
+            std::cerr << thread_id << "Cannot stat file m_time for " << filename << ". stat() returned errno EACCES." << std::endl;
 #endif
             syslog(LOG_ERR, "Error reading %s. Check directory and file permissions and ownership. They should be 640 and 750 and readable by the e2guardian user: %s", filename, strerror(errno));
             return 0;
         }
         else {
             if (!is_daemonised) {
-                std::cerr << "Error reading " << filename << "Check directory and file permissions and ownership. They should be 750 and 640 and readable by the e2guardian user: " << strerror(errno) << std::endl;
+                std::cerr << thread_id << "Error reading " << filename << "Check directory and file permissions and ownership. They should be 750 and 640 and readable by the e2guardian user: " << strerror(errno) << std::endl;
             }
             syslog(LOG_ERR, "Error reading %s. Check directory and file permissions and ownership. They should be 750 and 640 and readable by the e2guardian user: %s", filename, strerror(errno));
             return 0;
@@ -1726,6 +1663,9 @@ time_t getFileDate(const char *filename)
 
 bool ListContainer::upToDate()
 {
+    if (sourcefile.startsWith("memory:"))
+        return true;
+
     if (getFileDate(sourcefile.toCharArray()) > filedate) {
         return false;
     }
@@ -1741,7 +1681,7 @@ bool ListContainer::upToDate()
 bool ListContainer::readTimeTag(String *tag, TimeLimit &tl)
 {
 #ifdef DGDEBUG
-    std::cout << "Found a time tag" << std::endl;
+    std::cerr << thread_id << "Found a time tag" << std::endl;
 #endif
     unsigned int tsthour, tstmin, tendhour, tendmin;
     String temp((*tag).after("#time: "));
@@ -1756,35 +1696,35 @@ bool ListContainer::readTimeTag(String *tag, TimeLimit &tl)
     tdays.removeWhiteSpace();
     if (tsthour > 23) {
         if (!is_daemonised) {
-            std::cerr << "Time Tag Start Hour over bounds." << std::endl;
+            std::cerr << thread_id << "Time Tag Start Hour over bounds." << std::endl;
         }
         syslog(LOG_ERR, "%s", "Time Tag Start Hour over bounds.");
         return false;
     }
     if (tendhour > 23) {
         if (!is_daemonised) {
-            std::cerr << "Time Tag End Hour over bounds." << std::endl;
+            std::cerr << thread_id << "Time Tag End Hour over bounds." << std::endl;
         }
         syslog(LOG_ERR, "%s", "Time Tag End Hour over bounds.");
         return false;
     }
     if (tstmin > 59) {
         if (!is_daemonised) {
-            std::cerr << "Time Tag Start Min over bounds." << std::endl;
+            std::cerr << thread_id << "Time Tag Start Min over bounds." << std::endl;
         }
         syslog(LOG_ERR, "%s", "Time Tag Start Min over bounds.");
         return false;
     }
     if (tendmin > 59) {
         if (!is_daemonised) {
-            std::cerr << "Time Tag End Min over bounds." << std::endl;
+            std::cerr << thread_id << "Time Tag End Min over bounds." << std::endl;
         }
         syslog(LOG_ERR, "%s", "Time Tag End Min over bounds.");
         return false;
     }
     if (tdays.length() > 7) {
         if (!is_daemonised) {
-            std::cerr << "Time Tag Days over bounds." << std::endl;
+            std::cerr << thread_id << "Time Tag Days over bounds." << std::endl;
         }
         syslog(LOG_ERR, "%s", "Time Tag Days over bounds.");
         return false;
@@ -1852,7 +1792,7 @@ bool ListContainer::isNow(int index)
         }
     }
 #ifdef DGDEBUG
-    std::cout << "time match " << tl.sthour << ":" << tl.stmin << "-" << tl.endhour << ":" << tl.endmin << " " << hour << ":" << min << " " << sourcefile << std::endl;
+    std::cerr << thread_id << "time match " << tl.sthour << ":" << tl.stmin << "-" << tl.endhour << ":" << tl.endmin << " " << hour << ":" << min << " " << sourcefile << std::endl;
 #endif
     return true;
 }
@@ -1862,7 +1802,7 @@ int ListContainer::getCategoryIndex(String *lcat)
     // where in the category list is our category? if nowhere, add it.
     if ((*lcat).length() < 2) {
 #ifdef DGDEBUG
-        std::cout << "blank entry index" << std::endl;
+        std::cerr << thread_id << "blank entry index" << std::endl;
 #endif
         return 0; // blank entry index
     }
@@ -1899,3 +1839,50 @@ String ListContainer::getListCategoryAtD(int index)
     }
     return listcategory[index];
 }
+
+
+// search for IP in list of individual IPs, ranges, subnets
+const char* ListContainer::inIPList(const std::string &ipstr )
+{
+    struct in_addr addr;
+    inet_aton(ipstr.c_str(), &addr);
+    uint32_t ip = ntohl(addr.s_addr);
+    // start with individual IPs
+    if (std::binary_search(iplist.begin(), iplist.end(), ip)) {
+        // only return a hostname if that's what we matched against
+        return "";
+    }
+
+    // ranges
+    for (std::list<ipl_rangestruct>::const_iterator i = iprangelist.begin(); i != iprangelist.end(); ++i) {
+        if ((ip >= i->startaddr) && (ip <= i->endaddr)) {
+            String ret = hIPtoChar(i->startaddr);
+            ret += "-";
+            ret += hIPtoChar(i->endaddr);
+//            return ret;
+            return "";
+        }
+    }
+
+    // subnets
+    for (std::list<ipl_subnetstruct>::const_iterator i = ipsubnetlist.begin(); i != ipsubnetlist.end(); ++i) {
+        if (i->maskedaddr == (ip & i->mask)) {
+            String ret = hIPtoChar(i->maskedaddr);
+            ret += "/";
+            ret += hIPtoChar(i->mask);
+            //return ret;
+            return "";
+        }
+    }
+
+#ifdef DGDEBUG
+    std::cerr << thread_id << "inIPList no match for " << ipstr << std::endl;
+#endif
+    return NULL;
+}
+
+const char *ListContainer::hIPtoChar(uint32_t ip) {
+    struct in_addr addr;
+    addr.s_addr = htonl(ip);
+    return inet_ntoa(addr);
+}
