@@ -23,19 +23,20 @@
 // GLOBALS
 
 extern bool is_daemonised;
+extern thread_local std::string thread_id;
 
 
 
-ListContainer total_block_site_list;
-ListContainer total_block_url_list;
+//ListContainer total_block_site_list;
+//ListContainer total_block_url_list;
 
 // IMPLEMENTATION
 
 OptionContainer::OptionContainer()
-    : use_filter_groups_list(false), stats_human_readable(false), auth_requires_user_and_group(false), use_group_names_list(false), auth_needs_proxy_query(false), prefer_cached_lists(false), no_daemon(false), no_logger(false), log_syslog(false), anonymise_logs(false), log_ad_blocks(false), log_timestamp(false), log_user_agent(false), soft_restart(false), delete_downloaded_temp_files(false), max_logitem_length(2000), max_content_filter_size(0), max_content_ramcache_scan_size(0), max_content_filecache_scan_size(0), scan_clean_cache(0), content_scan_exceptions(0), initial_trickle_delay(0), trickle_delay(0), content_scanner_timeout(0), reporting_level(0), weighted_phrase_mode(0), numfg(0), dstat_log_flag(false), dstat_interval(300), dns_user_logging(false), LC_cnt(0)
+    : use_filter_groups_list(false), stats_human_readable(false), auth_requires_user_and_group(false), use_group_names_list(false), auth_needs_proxy_query(false), prefer_cached_lists(false), e2_front_log(false), no_daemon(false), no_logger(false), log_syslog(false), anonymise_logs(false), log_ad_blocks(false), log_timestamp(false), log_user_agent(false), soft_restart(false), delete_downloaded_temp_files(false), max_logitem_length(2000), max_content_filter_size(0), max_content_ramcache_scan_size(0), max_content_filecache_scan_size(0), scan_clean_cache(0), content_scan_exceptions(0), initial_trickle_delay(0), trickle_delay(0), content_scanner_timeout(0), reporting_level(0), weighted_phrase_mode(0), numfg(0), dstat_log_flag(false), dstat_interval(300), dns_user_logging(false), LC_cnt(0), abort_on_missing_list(false), SB_trace(false), default_fg(0), default_trans_fg(0), default_icap_fg(0)
 {
     log_Q = new Queue<std::string>;
-    http_worker_Q = new Queue<Socket*>;
+   // http_worker_Q = new Queue<LQ_rec>;
 }
 
 
@@ -123,7 +124,17 @@ bool OptionContainer::read(std::string& filename, int type)
                         log_location += "/access.log";
                         log_syslog = false;
                     }
+#ifndef NEWDEBUG_OFF
+		    if((debuglevel = findoptionS("debuglevel")) != "")
+		    {
 
+		    }
+		    if((path_debuglevel = findoptionS("debuglevelfile")) != "")
+		    {
+
+		    }
+		    myDebug = new DebugManager(debuglevel, path_debuglevel);
+#endif
                     if ((stat_location = findoptionS("statlocation")) == "") {
                         stat_location = __LOGLOCATION;
                         stat_location += "/stats";
@@ -172,6 +183,14 @@ bool OptionContainer::read(std::string& filename, int type)
 		} else {
 			no_daemon = false;
 		}
+		
+		if (findoptionS("dockermode") == "on") {
+			no_daemon = true;
+			e2_front_log = true;
+		} else {
+			no_daemon = false;
+			e2_front_log = false;
+		}
 
 		if (findoptionS("nologger") == "on") {
 			no_logger = true;
@@ -272,7 +291,7 @@ bool OptionContainer::read(std::string& filename, int type)
 
         max_header_lines = findoptionI("maxheaderlines");
         if (max_header_lines == 0)
-            max_header_lines= 40;
+            max_header_lines = 40;
         if (!realitycheck(max_header_lines, 10, 250, "maxheaderlines")) {
             return false;
         }
@@ -286,6 +305,22 @@ bool OptionContainer::read(std::string& filename, int type)
             return false;
         }
 
+        connect_timeout_sec = findoptionI("connecttimeout");
+        if (connect_timeout_sec == 0)
+            connect_timeout_sec = 3;
+        if (!realitycheck(connect_timeout_sec, 1, 100, "connecttimeout")) {
+            return false;
+        } // check its a reasonable value
+        connect_timeout = connect_timeout_sec * 1000;
+
+        connect_retries = findoptionI("connectretries");
+        if (connect_retries == 0)
+            connect_retries  = 5;
+        if (!realitycheck(connect_retries, 1, 100, "connectretries")) {
+            return false;
+        } // check its a reasonable value
+
+
         proxy_timeout_sec = findoptionI("proxytimeout");
         if (!realitycheck(proxy_timeout_sec, 5, 100, "proxytimeout")) {
             return false;
@@ -410,7 +445,13 @@ bool OptionContainer::read(std::string& filename, int type)
             if (!realitycheck(content_scanner_timeout_sec, 1, 0, "contentscannertimeout")) {
                 return false;
             }
-            content_scanner_timeout = content_scanner_timeout_sec * 1000;
+
+            if (content_scanner_timeout_sec > 0)
+                content_scanner_timeout = content_scanner_timeout_sec * 1000;
+            else {
+                content_scanner_timeout = pcon_timeout;
+                content_scanner_timeout_sec = pcon_timeout_sec;
+            }
 
             if (findoptionS("scancleancache") == "off") {
                 scan_clean_cache = false;
@@ -431,6 +472,13 @@ bool OptionContainer::read(std::string& filename, int type)
             delete_downloaded_temp_files = true;
         }
 
+        if (findoptionS("searchsitelistforip"
+                                "") == "off") {
+            search_sitelist_for_ip = false;
+        } else {
+            search_sitelist_for_ip = true;
+        }
+
         url_cache_number = findoptionI("urlcachenumber");
         if (!realitycheck(url_cache_number, 0, 0, "urlcachenumber")) {
             return false;
@@ -488,11 +536,18 @@ bool OptionContainer::read(std::string& filename, int type)
             banned_flash.read(custom_banned_flash_file.c_str());
         }
 
-        proxy_port = findoptionI("proxyport");
-        if (!realitycheck(proxy_port, 1, 65535, "proxyport")) {
-            return false;
-        } // etc
         proxy_ip = findoptionS("proxyip");
+        if (proxy_ip == "")
+            no_proxy = true;
+        else
+            no_proxy = false;
+
+        if (!no_proxy) {
+            proxy_port = findoptionI("proxyport");
+            if (!realitycheck(proxy_port, 1, 65535, "proxyport")) {
+                return false;
+            } // etc
+        }
 
         // multiple listen IP support
         filter_ip = findoptionM("filterip");
@@ -516,6 +571,24 @@ bool OptionContainer::read(std::string& filename, int type)
             return false;
         } // check its a reasonable value
 
+        transparenthttps_port = findoptionI("transparenthttpsport");
+        if (!realitycheck(filter_port, 0, 65535, "transparenthttpsport")) {
+            return false;
+        } // check its a reasonable value
+
+        icap_port = findoptionI("icapport");
+        if (!realitycheck(filter_port, 0, 65535, "icapport")) {
+            return false;
+        } // check its a reasonable value
+
+        icap_reqmod_url = findoptionS("icapreqmodurl");
+        if (icap_reqmod_url == "")
+            icap_reqmod_url = "request";
+
+        icap_resmod_url = findoptionS("icapresmodurl");
+        if (icap_resmod_url == "")
+            icap_resmod_url = "response";
+
 #ifdef ENABLE_ORIG_IP
         if (findoptionS("originalip") == "on") {
             get_orig_ip = true;
@@ -588,11 +661,7 @@ bool OptionContainer::read(std::string& filename, int type)
         if (!realitycheck(log_exception_hits, 0, 2, "logexceptionhits")) {
             return false;
         }
-        if (findoptionS("createlistcachefiles") == "off") {
-            createlistcachefiles = false;
-        } else {
-            createlistcachefiles = true;
-        }
+
         if (findoptionS("logconnectionhandlingerrors") == "on") {
             logconerror = true;
         } else {
@@ -622,17 +691,11 @@ bool OptionContainer::read(std::string& filename, int type)
         }
         if (findoptionS("logclienthostnames") == "on") {
             log_client_hostnames = true;
+            reverse_client_ip_lookups = true;
         } else {
             log_client_hostnames = false;
         }
-	
-	if (log_client_hostnames == true && reverse_client_ip_lookups == false){
-            if (!is_daemonised) {
-                std::cerr << "logclienthostnames enabled but reverseclientiplookups disabled" << std::endl;
-            }
-            syslog(LOG_ERR, "logclienthostnames enabled but reverseclientiplookups disabled");
-            return false;
-	}
+
         if (findoptionS("recheckreplacedurls") == "on") {
             recheck_replaced_urls = true;
         } else {
@@ -649,6 +712,52 @@ bool OptionContainer::read(std::string& filename, int type)
 
         filter_groups = findoptionI("filtergroups");
 
+        default_fg = findoptionI("defaultfiltergroup");
+        if (default_fg > 0) {
+            if (default_fg <= filter_groups) {
+                default_fg--;
+            } else  {
+                syslog(LOG_ERR, "defaultfiltergroup out of range");
+                return false;
+            }
+        }
+
+        default_trans_fg = findoptionI("defaulttransparentfiltergroup");
+        if (default_trans_fg > 0) {
+            if (default_trans_fg <= filter_groups) {
+                default_trans_fg--;
+            } else  {
+                syslog(LOG_ERR, "defaulttransparentfiltergroup out of range");
+                return false;
+            }
+        }
+
+        default_icap_fg = findoptionI("defaulticapfiltergroup");
+        if (default_icap_fg > 0) {
+            if (default_icap_fg <= filter_groups) {
+                default_icap_fg--;
+            } else  {
+                syslog(LOG_ERR, "defaulticapfiltergroup out of range");
+                return false;
+            }
+        }
+
+        if (findoptionS("abortiflistmissing") == "on")
+        {
+            abort_on_missing_list = true;
+        } else {
+            abort_on_missing_list = false;
+        }
+
+        if (findoptionS("storyboardtrace") == "on")
+        {
+            SB_trace = true;
+        } else {
+            SB_trace = false;
+        }
+
+        storyboard_location = findoptionS("preauthstoryboard");
+
         if (((per_room_directory_location = findoptionS("perroomdirectory")) != "") || ((per_room_directory_location = findoptionS("perroomblockingdirectory")) != "")) {
             //loadRooms(true);
         }
@@ -660,7 +769,6 @@ bool OptionContainer::read(std::string& filename, int type)
             if (!is_daemonised) {
                 std::cerr << "filtergroups too small" << std::endl;
             }
-            syslog(LOG_ERR, "filtergroups too small");
             return false;
         }
 
@@ -717,8 +825,6 @@ bool OptionContainer::read(std::string& filename, int type)
         if (authplugins.size() > 1 and filter_ports.size() > 1 and map_auth_to_ports) {
             std::deque<Plugin *>::iterator it = authplugins.begin();
             String firstPlugin;
-            bool sslused = false;
-            bool coreused = false;
             while (it != authplugins.end()) {
                 AuthPlugin *tmp = (AuthPlugin *)*it;
                 if (tmp->getPluginName().startsWith("proxy-basic")) {
@@ -747,15 +853,20 @@ bool OptionContainer::read(std::string& filename, int type)
             }
         }
 
-        // if there's no auth enabled, we only need the first group's settings
-        if (authplugins.size() == 0)
-            filter_groups = 1;
+        // if there's no auth enabled, we only need the first group's settings - THIS IS NO LONGER THE CASE TRANs, ICAP canbe set to different defaults
+        //if (authplugins.size() == 0)
+        //    filter_groups = 1;
+        numfg = filter_groups;
 
         filter_groups_list_location = findoptionS("filtergroupslist");
         banned_ip_list_location = findoptionS("bannediplist");
         exception_ip_list_location = findoptionS("exceptioniplist");
         group_names_list_location = findoptionS("groupnamesfile");
         std::string language_list_location(languagepath + "messages");
+        iplist_dq = findoptionM("iplist");
+        sitelist_dq = findoptionM("sitelist");
+        ipsitelist_dq = findoptionM("ipsitelist");
+        urllist_dq = findoptionM("urllist");
 
         if (filter_groups_list_location.length() == 0) {
             use_filter_groups_list = false;
@@ -777,43 +888,21 @@ bool OptionContainer::read(std::string& filename, int type)
             use_group_names_list = true;
         }
 
-        if (findoptionS("prefercachedlists") == "on")
-            prefer_cached_lists = true;
-        else
-            prefer_cached_lists = false;
 
-        //if (!exception_ip_list.readIPMelangeList(exception_ip_list_location.c_str())) {
-        //    std::cout << "Failed to read exceptioniplist" << std::endl;
-        //    return false;
-        //}
-        //if (!banned_ip_list.readIPMelangeList(banned_ip_list_location.c_str())) {
-        //    std::cout << "Failed to read bannediplist" << std::endl;
-        //    return false;
-        //}
 
         if (!language_list.readLanguageList(language_list_location.c_str())) {
             return false;
         } // messages language file
 
 
-        if(!createLists(0))  {
-                std::cerr << "Error reading filter group conf file(s)." << std::endl;
-            syslog(LOG_ERR, "%s", "Error reading filter group conf file(s).");
-            return false;
-        }
+ //       if(!createLists(0))  {
+ //               std::cerr << "Error reading filter group conf file(s)." << std::endl;
+//            syslog(LOG_ERR, "%s", "Error reading filter group conf file(s).");
+//            return false;
+//        }
 
-//post read filtergroup config checks - only for SLLMITM for now
 
 #ifdef _SSLMITM
-        //bool ssl_mitm = false;
-        //bool mitm_check_cert = false;
-        //for (i = 0; i < numfg; i++) {
-            //if (fg[i].ssl_mitm)
-                //ssl_mitm = true;
-            //if (fg[i].mitm_check_cert)
-                //mitm_check_cert = true;
-        //}
-
         if (enable_ssl) {
             if (ca_certificate_path != "") {
                 ca = new CertificateAuthority(ca_certificate_path.c_str(),
@@ -844,9 +933,10 @@ bool OptionContainer::read(std::string& filename, int type)
 // read from stdin, write the list's ID into the given identifier,
 // sort using startsWith or endsWith depending on sortsw
 // listname is used in error messages.
-bool OptionContainer::readStdin(ListContainer *lc, bool sortsw, const char *listname, const char *startstr)
+#ifdef NOTDEF
+bool OptionContainer::readStdin(ListContainer *lc, bool sortsw, const char *listname )
 {
-    bool result = lc->readStdinItemList(sortsw, 1, startstr);
+    bool result = lc->readStdinItemList(sortsw, 1);
     if (!result) {
         if (!is_daemonised) {
             std::cerr << "Error opening " << listname << std::endl;
@@ -860,23 +950,77 @@ bool OptionContainer::readStdin(ListContainer *lc, bool sortsw, const char *list
         lc->doSort(false);
     return true;
 }
+#endif
 
 bool OptionContainer::readinStdin()
 {
-    String sitelist = "totalblocksitelist";
-    String sitess = "#SITELIST";
-    if (!readStdin(&total_block_site_list, false, sitelist.c_str(), sitess.c_str())) {
+    if (!std::cin.good()) {
+        if (!is_daemonised) {
+            std::cerr << thread_id << "Error reading stdin: " << std::endl;
+        }
+        syslog(LOG_ERR, "Error reading stdin");
         return false;
     }
-    total_block_site_flag = true;
-    if (!readStdin(&total_block_url_list, true, "totalblockurllist", "#URLLIST")) {
-        return false;
+    std::string linebuffer;
+    String temp ;
+    while (!std::cin.eof()) {
+    //   std::cerr << "wiating for stdin" << std::endl;
+        getline(std::cin, linebuffer);
+    //    std::cerr << "Line in: " << linebuffer << std::endl;
+        if (linebuffer.length() < 2)
+            continue; // its jibberish
+
+        temp = linebuffer.c_str();
+        bool site_list = false;
+        bool url_list = false;
+        if (linebuffer[0] == '#') {
+            if (temp.startsWith("#SITELIST:"))
+                site_list = true;
+            else if (temp.startsWith("#URLLIST:"))
+                url_list = true;
+            else
+                continue;
+            String param = temp.after(":");
+            String nm, fpath;
+            String t = param;
+            bool startswith;
+            t.removeWhiteSpace();
+            t = t + ",";
+            while (t.length() > 0) {
+                if (t.startsWith("name=")) {
+                    nm = t.after("=").before(",");
+                } else if (t.startsWith("path=")) {
+                    fpath = t.after("=").before(",");
+                }
+                t = t.after(",");
+            }
+            if (!fpath.startsWith("memory:")) {
+                // syntax error
+                return false;
+            }
+            if (nm.length() == 0) {
+                // syntax error
+                return false;
+            }
+            if(site_list)
+                startswith = false;
+            else
+                startswith = true;
+
+            int rc = lm.newItemList(fpath.c_str(),startswith,1, true);
+            if (rc < 0)
+                return false;
+             lm.l[rc]->doSort(url_list);
+            if(site_list)
+                sitelist_dq.push_back(param);
+            else
+                urllist_dq.push_back(param);
+        }
     }
-    total_block_url_flag = true;
     return true;
 }
 
-char *OptionContainer::inSiteList(String &url, ListContainer *lc, bool ip, bool ssl)
+const char *OptionContainer::inSiteList(String &url, ListContainer *lc, bool ip, bool ssl)
 {
     String lastcategory;
     url.removeWhiteSpace(); // just in case of weird browser crap
@@ -885,7 +1029,7 @@ char *OptionContainer::inSiteList(String &url, ListContainer *lc, bool ip, bool
     if (url.contains("/")) {
         url = url.before("/"); // chop off any path after the domain
     }
-    char *i;
+    const char *i;
     //bool isipurl = isIPHostname(url);
     while (url.contains(".")) {
         i = lc->findInList(url.toCharArray(), lastcategory);
@@ -942,7 +1086,7 @@ char *OptionContainer::inURLList(String &url, ListContainer *lc, bool ip, bool s
             std::cout << "foundurl: " << foundurl << foundurl.length() << std::endl;
             std::cout << "url: " << url << fl << std::endl;
 #endif
-            //syslog(LOG_ERR, "inURLList foundurl  %s", foundurl.c_str());
+            ///syslog(LOG_ERR, "inURLList foundurl  %s", foundurl.c_str());
             if (url.length() > fl) {
                 if (url[fl] == '/' || url[fl] == '?' || url[fl] == '&' || url[fl] == '=') {
                     return i; // matches /blah/ or /blah/foo but not /blahfoo
@@ -956,6 +1100,7 @@ char *OptionContainer::inURLList(String &url, ListContainer *lc, bool ip, bool s
     return NULL;
 }
 
+#ifdef NOTDEF
 bool OptionContainer::inTotalBlockList(String &url)
 {
     String murl = url;
@@ -968,7 +1113,10 @@ bool OptionContainer::inTotalBlockList(String &url)
     }
     return false;
 }
+#endif
 
+
+#ifdef NOTDEF
 bool OptionContainer::doReadItemList(const char *filename, ListContainer *lc, const char *fname, bool swsort)
 {
     bool result = lc->readItemList(filename, false, 0);
@@ -985,6 +1133,7 @@ bool OptionContainer::doReadItemList(const char *filename, ListContainer *lc, co
         lc->doSort(false);
     return true;
 }
+#endif
 
 
 long int OptionContainer::findoptionI(const char *option)
