@@ -8,6 +8,7 @@
 #ifdef HAVE_CONFIG_H
 #include "dgconfig.h"
 #endif
+
 #include "Socket.hpp"
 
 #include <string.h>
@@ -40,11 +41,17 @@ extern bool reloadconfig;
 #endif
 #endif
 
+extern thread_local std::string thread_id;
+
 // IMPLEMENTATION
+//
+// destructor
+Socket::~Socket() {
+    close();
+}
 
 // constructor - create an INET socket & clear address structs
-Socket::Socket()
-{
+Socket::Socket() {
     sck = socket(AF_INET, SOCK_STREAM, 0);
     if (sck < 0) {
         s_errno = errno;
@@ -60,6 +67,7 @@ Socket::Socket()
             int res = setsockopt(sck, IPPROTO_TCP, TCP_NODELAY, &f, sizeof(int));
 
         my_port = 0;
+        chunkError = false;
 
 #ifdef __SSLMITM
         ssl = NULL;
@@ -74,8 +82,7 @@ Socket::Socket()
 
 // create socket from pre-existing FD (address structs will be invalid!)
 Socket::Socket(int fd)
-    : BaseSocket(fd)
-{
+        : BaseSocket(fd) {
     memset(&my_adr, 0, sizeof my_adr);
     memset(&peer_adr, 0, sizeof peer_adr);
     my_adr.sin_family = AF_INET;
@@ -86,6 +93,7 @@ Socket::Socket(int fd)
     int res = setsockopt(sck, IPPROTO_TCP, TCP_NODELAY, &f, sizeof(int));
     if (res < 0) s_errno = errno;
     my_port = 0;
+    chunkError = false;
 
 #ifdef __SSLMITM
     ssl = NULL;
@@ -99,8 +107,7 @@ Socket::Socket(int fd)
 
 // create socket from pre-existing FD, storing local & remote IPs
 Socket::Socket(int newfd, struct sockaddr_in myip, struct sockaddr_in peerip)
-    : BaseSocket(newfd)
-{
+        : BaseSocket(newfd) {
     memset(&my_adr, 0, sizeof my_adr); // ***
     memset(&peer_adr, 0, sizeof peer_adr); // ***
     my_adr.sin_family = AF_INET; // *** Fix suggested by
@@ -113,6 +120,7 @@ Socket::Socket(int newfd, struct sockaddr_in myip, struct sockaddr_in peerip)
     int res = setsockopt(sck, IPPROTO_TCP, TCP_NODELAY, &f, sizeof(int));
     if (res < 0) s_errno = errno;
     my_port = 0;
+    chunkError = false;
 
 #ifdef __SSLMITM
     ssl = NULL;
@@ -125,40 +133,37 @@ Socket::Socket(int newfd, struct sockaddr_in myip, struct sockaddr_in peerip)
 }
 
 // find the ip to which the client has connected
-std::string Socket::getLocalIP()
-{
-    return inet_ntoa(my_adr.sin_addr);
+std::string Socket::getLocalIP() {
+    char res[INET_ADDRSTRLEN];
+    return inet_ntop(AF_INET,&my_adr.sin_addr, res, sizeof(res));
 }
 
 // find the ip of the client connecting to us
-std::string Socket::getPeerIP()
-{
-    return inet_ntoa(peer_adr.sin_addr);
+std::string Socket::getPeerIP() {
+    char res[INET_ADDRSTRLEN];
+    return inet_ntop(AF_INET,&peer_adr.sin_addr, res, sizeof(res));
 }
 
 // find the port of the client connecting to us
-int Socket::getPeerSourcePort()
-{
+int Socket::getPeerSourcePort() {
     return ntohs(peer_adr.sin_port);
 }
-int Socket::getPort()
-{
+
+int Socket::getPort() {
     return my_port;
 }
-void Socket::setPort(int port)
-{
+
+void Socket::setPort(int port) {
     my_port = port;
 }
 
 // return the address of the client connecting to us
-unsigned long int Socket::getPeerSourceAddr()
-{
-    return (unsigned long int)ntohl(peer_adr.sin_addr.s_addr);
+unsigned long int Socket::getPeerSourceAddr() {
+    return (unsigned long int) ntohl(peer_adr.sin_addr.s_addr);
 }
 
 // close connection & wipe address structs
-void Socket::reset()
-{
+void Socket::reset() {
     this->baseReset();
 
     sck = socket(AF_INET, SOCK_STREAM, 0);
@@ -175,6 +180,9 @@ void Socket::reset()
     infds[0].fd = sck;
     outfds[0].fd = sck;
 
+    chunkError = false;
+    chunk_to_read = 0;
+
 #ifdef __SSLMITM
     if (isssl) {
         stopSsl();
@@ -183,13 +191,12 @@ void Socket::reset()
 }
 
 // connect to given IP & port (following default constructor)
-int Socket::connect(const std::string &ip, int port)
-{
-    //if (sck > -1)  reset();   // just in case this is called with socket still open
+int Socket::connect(const std::string &ip, int port) {
     reset();   // do it anyway as we need sck to be allocated
 
     if (sck < 0) // socket creation error
-    { return -1;
+    {
+        return -1;
     }
 
     int len = sizeof my_adr;
@@ -200,20 +207,18 @@ int Socket::connect(const std::string &ip, int port)
     fcntl(sck, F_SETFL, O_NONBLOCK);
     s_errno = 0;
     errno = 0;
-    int ret = ::connect(sck, (struct sockaddr *)&peer_adr, len);
+    int ret = ::connect(sck, (struct sockaddr *) &peer_adr, len);
     if (ret < 0 && errno == EINPROGRESS) ret = 0;
     else s_errno = errno;
-    if (ret == 0){
-        int rc = poll(outfds,1,timeout);
+    if (ret == 0) {
+        int rc = poll(outfds, 1, timeout);
         if (rc == 0) {
             timedout = true;
             ret = -1;
-        }
-        else if (rc < 0) {
+        } else if (rc < 0) {
             s_errno = errno;
             ret = -1;
-        }
-        else {  // ret == 1
+        } else {  // ret == 1
             int so_error;
             socklen_t len = sizeof so_error;
             getsockopt(sck, SOL_SOCKET, SO_ERROR, &so_error, &len);
@@ -230,9 +235,9 @@ int Socket::connect(const std::string &ip, int port)
     if (ret < 0) close();
     return ret;
 }
+
 // bind socket to given port
-int Socket::bind(int port)
-{
+int Socket::bind(int port) {
     int len = sizeof my_adr;
     int i = 1;
 
@@ -241,12 +246,11 @@ int Socket::bind(int port)
     my_adr.sin_port = htons(port);
     my_port = port;
 
-    return ::bind(sck, (struct sockaddr *)&my_adr, len);
+    return ::bind(sck, (struct sockaddr *) &my_adr, len);
 }
 
 // bind socket to given port & IP
-int Socket::bind(const std::string &ip, int port)
-{
+int Socket::bind(const std::string &ip, int port) {
     int len = sizeof my_adr;
     int i = 1;
 
@@ -256,17 +260,16 @@ int Socket::bind(const std::string &ip, int port)
     my_adr.sin_addr.s_addr = inet_addr(ip.c_str());
     my_port = port;
 
-    return ::bind(sck, (struct sockaddr *)&my_adr, len);
+    return ::bind(sck, (struct sockaddr *) &my_adr, len);
 }
 
 // accept incoming connections & return new Socket
-Socket *Socket::accept()
-{
+Socket *Socket::accept() {
     peer_adr_length = sizeof(struct sockaddr_in);
     s_errno = 0;
     errno = 0;
 //    int newfd = this->baseAccept((struct sockaddr *)&peer_adr, &peer_adr_length);
-    int newfd = ::accept(sck,( struct sockaddr *)&peer_adr, &peer_adr_length);
+    int newfd = ::accept(sck, (struct sockaddr *) &peer_adr, &peer_adr_length);
 
     if (newfd > 0) {
         Socket *s = new Socket(newfd, my_adr, peer_adr);
@@ -292,9 +295,9 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
 //needed to get the errors when creating ctx
 //ERR_print_errors_fp(stderr);
 //printerr(strerror(errno));
-#ifdef DGDEBUG
+#ifdef NETDEBUG
         //syslog(LOG_ERR, "error creating ssl context\n");
-        std::cout << "Error ssl context is null (check that openssl has been inited)" << std::endl;
+        std::cout << thread_id << "Error ssl context is null (check that openssl has been inited)" << std::endl;
 #endif
         log_ssl_errors("Error ssl context is null for %s", hostname.c_str());
         return -1;
@@ -311,8 +314,8 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
     ERR_clear_error();
     if (certificate_path.length()) {
         if (!SSL_CTX_load_verify_locations(ctx, NULL, certificate_path.c_str())) {
-#ifdef DGDEBUG
-            std::cout << "couldnt load certificates" << std::endl;
+#ifdef NETDEBUG
+            std::cout << thread_id << "couldnt load certificates" << std::endl;
 #endif
             log_ssl_errors("couldnt load certificates from %s", certificate_path.c_str());
             //tidy up
@@ -322,8 +325,8 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
         }
     } else if (!SSL_CTX_set_default_verify_paths(ctx)) //use default if no certPpath given
     {
-#ifdef DGDEBUG
-        std::cout << "couldnt load certificates" << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "couldnt load certificates" << std::endl;
 #endif
             log_ssl_errors("couldnt load default certificates for %s", hostname.c_str());
         //tidy up
@@ -382,8 +385,8 @@ int Socket::startSslClient(const std::string &certificate_path, String hostname)
     if (rc < 0) {
         //ERR_print_errors_fp(stderr);
         log_ssl_errors("ssl_connect failed to %s", hostname.c_str());
-#ifdef DGDEBUG
-        std::cout << "ssl_connect failed with error " << SSL_get_error(ssl, rc) << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "ssl_connect failed with error " << SSL_get_error(ssl, rc) << std::endl;
 #endif
         // tidy up
         SSL_free(ssl);
@@ -412,8 +415,8 @@ bool Socket::isSslServer()
 //shuts down the current ssl connection
 void Socket::stopSsl()
 {
-#ifdef DGDEBUG
-    std::cout << "ssl stopping" << std::endl;
+#ifdef NETDEBUG
+    std::cout << thread_id << "ssl stopping" << std::endl;
 #endif
     if(!isssl) return;
 
@@ -421,26 +424,26 @@ void Socket::stopSsl()
 
     if (ssl != NULL) {
         if (issslserver) {
-#ifdef DGDEBUG
-            std::cout << "this is a server connection" << std::endl;
+#ifdef NETDEBUG
+            std::cout << thread_id << "this is a server connection" << std::endl;
             if (SSL_get_shutdown(ssl) & SSL_SENT_SHUTDOWN) {
-                std::cout << "SSL_SENT_SHUTDOWN IS SET" << std::endl;
+                std::cout << thread_id << "SSL_SENT_SHUTDOWN IS SET" << std::endl;
             }
             if (SSL_get_shutdown(ssl) & SSL_RECEIVED_SHUTDOWN) {
-                std::cout << "SSL_RECIEVED_SHUTDOWN IS SET" << std::endl;
+                std::cout << thread_id << "SSL_RECIEVED_SHUTDOWN IS SET" << std::endl;
             }
-            std::cout << "calling 1st ssl shutdown" << std::endl;
+            std::cout << thread_id << "calling 1st ssl shutdown" << std::endl;
 #endif
             if (!SSL_shutdown(ssl)) {
-#ifdef DGDEBUG
-                std::cout << "need to call SSL shutdown again" << std::endl;
+#ifdef NETDEBUG
+                std::cout << thread_id << "need to call SSL shutdown again" << std::endl;
                 if (SSL_get_shutdown(ssl) & SSL_SENT_SHUTDOWN) {
-                    std::cout << "SSL_SENT_SHUTDOWN IS SET" << std::endl;
+                    std::cout << thread_id << "SSL_SENT_SHUTDOWN IS SET" << std::endl;
                 }
                 if (SSL_get_shutdown(ssl) & SSL_RECEIVED_SHUTDOWN) {
-                    std::cout << "SSL_RECIEVED_SHUTDOWN IS SET" << std::endl;
+                    std::cout << thread_id << "SSL_RECIEVED_SHUTDOWN IS SET" << std::endl;
                 }
-                std::cout << "Discarding extra data from client" << std::endl;
+                std::cout << thread_id << "Discarding extra data from client" << std::endl;
 #endif
 
                 //SSL_set_quiet_shutdown(ssl,1);
@@ -448,24 +451,24 @@ void Socket::stopSsl()
                 shutdown(SSL_get_fd(ssl), SHUT_WR);
                 char junk[1024];
                 readFromSocket(junk, sizeof(junk), 0, 5);
-#ifdef DGDEBUG
-                std::cout << "done" << std::endl;
+#ifdef NETDEBUG
+                std::cout << thread_id << "done" << std::endl;
 #endif
             }
         } else {
-#ifdef DGDEBUG
-            std::cout << "this is a client connection" << std::endl;
+#ifdef NETDEBUG
+            std::cout << thread_id << "this is a client connection" << std::endl;
             if (SSL_get_shutdown(ssl) & SSL_SENT_SHUTDOWN) {
-                std::cout << "SSL_SENT_SHUTDOWN IS SET" << std::endl;
+                std::cout << thread_id << "SSL_SENT_SHUTDOWN IS SET" << std::endl;
             }
             if (SSL_get_shutdown(ssl) & SSL_RECEIVED_SHUTDOWN) {
-                std::cout << "SSL_RECIEVED_SHUTDOWN IS SET" << std::endl;
+                std::cout << thread_id << "SSL_RECIEVED_SHUTDOWN IS SET" << std::endl;
             }
-            std::cout << "calling ssl shutdown" << std::endl;
+            std::cout << thread_id << "calling ssl shutdown" << std::endl;
 #endif
             SSL_shutdown(ssl);
-#ifdef DGDEBUG
-            std::cout << "done" << std::endl;
+#ifdef NETDEBUG
+            std::cout << thread_id << "done" << std::endl;
 #endif
         }
 
@@ -481,7 +484,7 @@ void Socket::stopSsl()
 }
 
 //check that everything in this certificate is correct appart from the hostname
-long Socket::checkCertValid()
+long Socket::checkCertValid(String &hostname)
 {
     //check we have a certificate
     X509 *peerCert = SSL_get_peer_certificate(ssl);
@@ -489,6 +492,16 @@ long Socket::checkCertValid()
         return -1;
     }
     X509_free(peerCert);
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#else
+// section for openssl1.1
+X509_VERIFY_PARAM *param;
+param = X509_VERIFY_PARAM_new() ;
+X509_VERIFY_PARAM_set1_host(param,hostname.c_str(), hostname.length());
+SSL_CTX_set1_param(ctx,param);
+X509_VERIFY_PARAM_free(param);
+#endif
     return SSL_get_verify_result(ssl);
 }
 
@@ -501,19 +514,22 @@ int Socket::checkCertHostname(const std::string &_hostname)
 
     X509 *peercertificate = SSL_get_peer_certificate(ssl);
     if (peercertificate == NULL) {
-#ifdef DGDEBUG
-        std::cout << "unable to get certificate for " << hostname << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "unable to get certificate for " << hostname << std::endl;
 #endif
         return -1;
     }
     //force to lower case as domain names are not case sensetive
     hostname.toLower();
 
-#ifdef DGDEBUG
-    std::cout << "checking certificate" << hostname << std::endl;
-    std::cout << "Checking hostname against subjectAltNames" << std::endl;
+#ifdef NETDEBUG
+    std::cout << thread_id << "checking certificate" << hostname << std::endl;
+    std::cout << thread_id << "Checking hostname against subjectAltNames" << std::endl;
 #endif
 
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+
     //check the altname extension for additional valid names
     STACK_OF(GENERAL_NAME) *gens = NULL;
     gens = (STACK_OF(GENERAL_NAME) *)X509_get_ext_d2i(peercertificate, NID_subject_alt_name, 0, 0);
@@ -542,16 +558,16 @@ int Socket::checkCertHostname(const std::string &_hostname)
         //force to lower case as domain names are not case sensetive
         altname.toLower();
 
-#ifdef DGDEBUG
-        std::cout << "checking against alt name " << altname << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "checking against alt name " << altname << std::endl;
 #endif
 
         if (hostname.compare(altname) == 0) {
             matched = true;
             break;
         } else if (altname.startsWith("*.")) {
-#ifdef DGDEBUG
-            std::cout << "Wildcard certificate is in use" << std::endl;
+#ifdef NETDEBUG
+            std::cout << thread_id << "Wildcard certificate is in use" << std::endl;
 #endif
             altname = altname.after("*"); // need to keep the "."
             if (hostname.endsWith(altname)) {
@@ -570,8 +586,8 @@ int Socket::checkCertHostname(const std::string &_hostname)
         return -1;
     }
 
-#ifdef DGDEBUG
-    std::cout << "checking hostname against the following common names" << std::endl;
+#ifdef NETDEBUG
+    std::cout << thread_id << "checking hostname against the following common names" << std::endl;
 #endif
 
     X509_NAME *name = X509_get_subject_name(peercertificate);
@@ -604,8 +620,8 @@ int Socket::checkCertHostname(const std::string &_hostname)
         //force to lower case as domain names are not case sensetive
         commonname.toLower();
 
-#ifdef DGDEBUG
-        std::cout << "checking against common name " << commonname << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "checking against common name " << commonname << std::endl;
 #endif
 
         //compare the hostname to the common name
@@ -615,8 +631,8 @@ int Socket::checkCertHostname(const std::string &_hostname)
         }
         //see if its a wildcard certificate
         else if (commonname.startsWith("*.")) {
-#ifdef DGDEBUG
-            std::cout << "Wildcard certificate is in use" << std::endl;
+#ifdef NETDEBUG
+            std::cout << thread_id << "Wildcard certificate is in use" << std::endl;
 #endif
             commonname = commonname.after("*"); // need to keep the "."
 
@@ -631,7 +647,9 @@ int Socket::checkCertHostname(const std::string &_hostname)
         X509_free(peercertificate);
         return 0;
     }
-
+#else  // is openssl v1.1 or above
+    return 0;    //TODO
+#endif
     return -1;
 }
 
@@ -659,9 +677,9 @@ int Socket::startSslServer(X509 *x, EVP_PKEY *privKey, std::string &set_cipher_l
     //setup the ssl server ctx
     ctx = SSL_CTX_new(SSLv23_server_method());
     if (ctx == NULL) {
-#ifdef DGDEBUG
+#ifdef NETDEBUG
         //syslog(LOG_ERR, "error creating ssl context\n");
-        std::cout << "Error ssl context is null (check that openssl has been inited)" << std::endl;
+        std::cout << thread_id << "Error ssl context is null (check that openssl has been inited)" << std::endl;
 #endif
         return -1;
     }
@@ -673,9 +691,9 @@ int Socket::startSslServer(X509 *x, EVP_PKEY *privKey, std::string &set_cipher_l
 
     //set the ctx to use the certificate
     if (SSL_CTX_use_certificate(ctx, x) < 1) {
-#ifdef DGDEBUG
+#ifdef NETDEBUG
         //syslog(LOG_ERR, "error creating ssl context\n");
-        std::cout << "Error using certificate" << std::endl;
+        std::cout << thread_id << "Error using certificate" << std::endl;
 #endif
         return -1;
     }
@@ -685,9 +703,9 @@ int Socket::startSslServer(X509 *x, EVP_PKEY *privKey, std::string &set_cipher_l
 
     //set the ctx to use the private key
     if (SSL_CTX_use_PrivateKey(ctx, privKey) < 1) {
-#ifdef DGDEBUG
+#ifdef NETDEBUG
         //syslog(LOG_ERR, "error creating ssl context\n");
-        std::cout << "Error using private key" << std::endl;
+        std::cout << thread_id << "Error using private key" << std::endl;
 #endif
         return -1;
     }
@@ -700,15 +718,22 @@ int Socket::startSslServer(X509 *x, EVP_PKEY *privKey, std::string &set_cipher_l
     SSL_set_accept_state(ssl);
 
     ERR_clear_error();
-    SSL_set_fd(ssl, this->getFD());
+    if(!SSL_set_fd(ssl, this->getFD())) {
+#ifdef NETDEBUG
+        std::cout << thread_id << "Error setting ssl fd connection" << std::endl;
+#endif
+        log_ssl_errors("ssl_set_fd failed to client %s", "");
+        stopSsl();
+        return -1;
+    };
 
     //make io non blocking as select wont tell us if we can do a read without blocking
 
     ERR_clear_error();
     if (SSL_accept(ssl) < 0) {
-#ifdef DGDEBUG
+#ifdef NETDEBUG
         //syslog(LOG_ERR, "error creating ssl context\n");
-        std::cout << "Error accepting ssl connection" << std::endl;
+        std::cout << thread_id << "Error accepting ssl connection" << std::endl;
 #endif
         log_ssl_errors("ssl_accept failed to client %s", "");
         stopSsl();
@@ -717,9 +742,9 @@ int Socket::startSslServer(X509 *x, EVP_PKEY *privKey, std::string &set_cipher_l
 
     ERR_clear_error();
     if (SSL_do_handshake(ssl) < 0) {
-#ifdef DGDEBUG
+#ifdef NETDEBUG
         //syslog(LOG_ERR, "error creating ssl context\n");
-        std::cout << "Error doing ssl handshake" << std::endl;
+        std::cout << thread_id << "Error doing ssl handshake" << std::endl;
 #endif
         log_ssl_errors("ssl_handshake failed to client %s", "");
         stopSsl();
@@ -739,12 +764,12 @@ bool Socket::checkForInput()
     if (!isssl) {
         return BaseSocket::checkForInput();
     }
-#ifdef DGDEBUG
-    std::cout << "checking for input on ssl connection (non blocking)" << std::endl;
+#ifdef NETDEBUG
+    std::cout << thread_id << "checking for input on ssl connection (non blocking)" << std::endl;
 #endif
     if ((bufflen - buffstart) > 0) {
-#ifdef DGDEBUG
-        std::cout << "found input on ssl connection" << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "found input on ssl connection" << std::endl;
 #endif
         return true;
     }
@@ -759,14 +784,14 @@ bool Socket::checkForInput()
     int rc = SSL_pending(ssl);
 
     if (rc < 1) {
-#ifdef DGDEBUG
-        std::cout << "no pending data on ssl connection SSL_pending " << rc << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "no pending data on ssl connection SSL_pending " << rc << std::endl;
 #endif
         return false;
     }
 
-#ifdef DGDEBUG
-    std::cout << "found data on ssl connection" << std::endl;
+#ifdef NETDEBUG
+    std::cout << thread_id << "found data on ssl connection" << std::endl;
 #endif
 
     return true;
@@ -802,22 +827,19 @@ bool Socket::breadyForOutput(int timeout)
 
 
 // read a line from the socket, can be told to break on config reloads
-int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig, bool *chopped, bool *truncated) throw(std::exception)
+int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig, bool *chopped, bool *truncated)
 {
+try {
     if (!isssl) {
         return BaseSocket::getLine(buff, size, timeout, honour_reloadconfig, chopped, truncated);
     }
-#ifdef DGDEBUG
-		std::cout << "in getline (SSL): timeout: " << timeout << " size: " << size << std::endl;
-
-#endif
 
     // first, return what's left from the previous buffer read, if anything
     int i = 0;
     if ((bufflen - buffstart) > 0) {
-#ifdef DGDEBUG
-		std::cout << "getline(ssl) data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
-#endif
+        /*#ifdef NETDEBUG
+        std::cout << thread_id << "data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
+#endif*/
         int tocopy = size - 1;
         if ((bufflen - buffstart) < tocopy)
             tocopy = bufflen - buffstart;
@@ -842,19 +864,16 @@ int Socket::getLine(char *buff, int size, int timeout, bool honour_reloadconfig,
  //       } catch (std::exception &e) {
 //            throw std::runtime_error(std::string("Can't read from socket: ") + strerror(errno)); // on error
  //       }
- //       if( bcheckSForInput(timeout))
+//        if( bcheckSForInput(timeout))
             bufflen = SSL_read(ssl, buffer, 4096);
-#ifdef DGDEBUG
-std::cout << "getline (SSL) read into buffer; bufflen: " << bufflen <<std::endl;
+#ifdef NETDEBUG
+//std::cout << thread_id << "read into buffer; bufflen: " << bufflen <<std::endl;
 #endif
         if (bufflen < 0) {
   //          if (errno == EINTR ) {
    //             continue;
     //        }
-
-#ifdef DGDEBUG
-            std::cerr << "SSL_read failed with error " << SSL_get_error(ssl, bufflen) << std::endl;
-#endif
+            std::cout << thread_id << "SSL_read failed with error " << SSL_get_error(ssl, bufflen) << std::endl;
             log_ssl_errors("ssl_read failed %s", "");
             return -1;
 //            throw std::runtime_error(std::string("Can't read from ssl socket")); //strerror(errno));  // on error
@@ -864,10 +883,6 @@ std::cout << "getline (SSL) read into buffer; bufflen: " << bufflen <<std::endl;
             buff[i] = '\0'; // ...terminate string & return what read
             if (truncated)
                 *truncated = true;
-#ifdef DGDEBUG
-            if (truncated)
-            	std::cout << "Getline(SSL) truncated buffer end reached before we found a newline: bufflen == 0" << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;;
-#endif
             return i;
         }
         int tocopy = bufflen;
@@ -875,9 +890,6 @@ std::cout << "getline (SSL) read into buffer; bufflen: " << bufflen <<std::endl;
             tocopy = (size - 1) - i;
         char *result = (char *)memccpy(buff + i, buffer, '\n', tocopy);
         if (result != NULL) {
-#ifdef DGDEBUG
-        std::cout << "getLine result (SSL): " << result << std::endl;
-#endif
             // indicate that a newline was chopped off, if desired
             if (chopped)
                 *chopped = true;
@@ -886,19 +898,14 @@ std::cout << "getline (SSL) read into buffer; bufflen: " << bufflen <<std::endl;
             return i + (result - (buff + i));
         }
         i += tocopy;
-#ifdef DGDEBUG
-            std::cout << "getLine(SSL) buffer =: " << buff[i] << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;
-#endif
     }
     // oh dear - buffer end reached before we found a newline
     buff[i] = '\0';
     if (truncated)
         *truncated = true;
-#ifdef DGDEBUG
-    	if (truncated)
-            std::cout << "Getline(SSL) truncated buffer end reached before we found a newline: " << buff  << " Line: " << __LINE__ << " Function: " << __func__ << std::endl;;
-#endif
     return i;
+    } catch (...) {
+    return -1;    }
 }
 
 // write line to socket
@@ -906,15 +913,20 @@ bool Socket::writeString(const char *line) //throw(std::exception)
 {
     int l = strlen(line);
     return writeToSocket(line, l, 0, timeout);
-//    if (!writeToSocket(line, l, 0, timeout)) {
- //       throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
-    //}
 }
 
+// write line to socket
+bool Socket::writeString(std::string line)
+{
+    int l = line.length();
+    return writeToSocket(line.c_str(), l, 0, timeout);
+}
 
 // write data to socket - can be told not to do an initial readyForOutput, and to break on config reloads
 bool Socket::writeToSocket(const char *buff, int len, unsigned int flags, int timeout, bool check_first, bool honour_reloadconfig)
 {
+    if (len == 0)   // nothing to write
+        return true;
     if (!isssl) {
         return BaseSocket::writeToSocket(buff, len, flags, timeout, check_first, honour_reloadconfig);
     }
@@ -937,7 +949,8 @@ bool Socket::writeToSocket(const char *buff, int len, unsigned int flags, int ti
     //            continue; // was interupted by signal so restart
     //        }
             s_errno = errno;
-            log_ssl_errors("ssl_write failed %s", "");
+            String serr(s_errno);
+            log_ssl_errors("ssl_write failed - error ",serr.c_str());
             return false;
         }
         if (sent == 0) {
@@ -965,8 +978,8 @@ int Socket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout
 
     // first, return what's left from the previous buffer read, if anything
     if ((bufflen - buffstart) > 0) {
-#ifdef DGDEBUG
-        std::cout << "Socket::readFromSocketn: data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "Socket::readFromSocketn: data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
 #endif
         int tocopy = len;
         if ((bufflen - buffstart) < len)
@@ -981,14 +994,14 @@ int Socket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout
 
     while (cnt > 0) {
     //    try {
-    //        bcheckSForInput(timeout);        //  this may be wrong - why is data not being read into socket buffer????
+            //bcheckSForInput(timeout);        //  this may be wrong - why is data not being read into socket buffer????
     //    } catch (std::exception &e) {
      //       return -1;
      //   }
         ERR_clear_error();
         rc = SSL_read(ssl, buff, cnt);
-#ifdef DGDEBUG
-        std::cout << "ssl read said: " << rc << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "ssl read said: " << rc << std::endl;
 #endif
 
         if (rc < 0) {
@@ -1013,6 +1026,8 @@ int Socket::readFromSocketn(char *buff, int len, unsigned int flags, int timeout
 // read what's available and return error status - can be told not to do an initial checkForInput, and to break on reloads
 int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout, bool check_first, bool honour_reloadconfig)
 {
+    if (len == 0)  // nothing to read
+         return 0;
     if (!isssl) {
         return BaseSocket::readFromSocket(buff, len, flags, timeout, check_first, honour_reloadconfig);
     }
@@ -1021,8 +1036,8 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
     int cnt = len;
     int tocopy = 0;
     if ((bufflen - buffstart) > 0) {
-#ifdef DGDEBUG
-        std::cout << "Socket::readFromSocket(ssl): data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "Socket::readFromSocket: data already in buffer; bufflen: " << bufflen << " buffstart: " << buffstart << std::endl;
 #endif
         tocopy = len;
         if ((bufflen - buffstart) < len)
@@ -1035,28 +1050,16 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
             return len;
     }
 
-#ifdef DGDEBUG
-        std::cout << "Socket::readFromSocket(ssl): now entering loop to read: " << cnt << " bytes: " << std::endl;
-#endif
     int rc;
     while (cnt > 0) {
- //   if (check_first) {
-//#ifdef DGDEBUG
-//       std::cout << "Socket::readFromSocket(ssl): before bcheckSForInput timeout is: " << timeout  << std::endl;
-//#endif
-//          if(!bcheckSForInput(timeout))
-  //          return -1;
-//#ifdef DGDEBUG
-//        std::cout << "Socket::readFromSocket(ssl): after bcheckSForInput OK return: " << std::endl;
-//#endif
-  // }
+    //if (check_first) {
+          //if(!bcheckSForInput(timeout))
+            //return -1;
+   //}
 //    while (true)
         bool inbuffer;
-#ifdef DGDEBUG
-        std::cout << "Socket::readFromSocket(ssl): before SSL_read: "  << std::endl;
-#endif
         ERR_clear_error();
-       if (true) {     // was if(cnt > 4095)
+        if(true) {   //   was if (cnt > 4095)
             inbuffer = false;
            rc = SSL_read(ssl, buff, cnt);        //  data larger than SSL buffer so ok to read directly into output buffer
         } else {
@@ -1067,8 +1070,8 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
         if (rc < 0) {
             s_errno = errno;
             log_ssl_errors("ssl_read failed %s", "");
-#ifdef DGDEBUG
-        std::cout << "ssl_read failed" << s_errno << " failed to read " << cnt << " bytes" << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "ssl_read failed" << s_errno << " failed to read " << cnt << " bytes" << std::endl;
 #endif
             rc = 0;
         }
@@ -1078,8 +1081,8 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
              }
 
         if (inbuffer) {
-#ifdef DGDEBUG
-        std::cout << "Inbuffer SSL read to return " << cnt << " bytes" << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "Inbuffer SSL read to return " << cnt << " bytes" << std::endl;
 #endif
 
            buffstart = 0;
@@ -1092,8 +1095,8 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
               cnt -= tocopy;
               buffstart += tocopy;
               buff += tocopy;
-#ifdef DGDEBUG
-        std::cout << "Inbuffer SSL read returned " << tocopy << " bytes" << std::endl;
+#ifdef NETDEBUG
+        std::cout << thread_id << "Inbuffer SSL read to returned " << tocopy << " bytes" << std::endl;
 #endif
            }
          } else {
@@ -1101,9 +1104,6 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
         cnt -= rc;
          }
  //       break;
-#ifdef DGDEBUG
-        std::cout << "Still to read (SSL) " << cnt << " bytes" << std::endl;
-#endif
     }
 
 //    return rc + tocopy;
@@ -1111,3 +1111,168 @@ int Socket::readFromSocket(char *buff, int len, unsigned int flags, int timeout,
 }
 
 #endif //__SSLMITM
+
+void Socket::resetChunk() {
+    chunk_to_read = 0;
+    chunked_trailer = "";
+    ieof = false;
+}
+
+bool Socket::writeChunk( char *buffout, int len, int timeout){
+    std::stringstream stm;
+    stm << std::hex << len;
+    std::string hexs (stm.str());
+    int lw;
+    hexs += "\r\n";
+#ifdef NETDEBUG
+    std::cerr << thread_id << "writeChunk  size=" << hexs << std::endl;
+#endif
+    if(writeString(hexs.c_str()) && writeToSocket(buffout,len,0,timeout) && writeString("\r\n"))
+        return true;
+    return false;
+};
+
+bool Socket::writeChunkTrailer( String &trailer) {
+    std::string hexs ("0\r\n");
+#ifdef CHUNKDEBUG
+    std::cerr << thread_id << "writeChunk  size=" << hexs << std::endl;
+#endif
+    if(writeString(hexs.c_str()) && writeToSocket(trailer.c_str(),trailer.length(),0,timeout) && writeString("\r\n"))
+        return true;
+    return false;
+};
+
+int Socket::readChunk( char *buffin, int maxlen, int timeout){
+    if (chunk_to_read == 0)     // need to read chunk size
+    {
+        char size[40];
+        ieof = false;
+        int len = getLine(size, 38, timeout);
+        if (len < 2) {   // min valid length is 2 i.e.  "0\r"
+            chunkError = true;
+            return -1;
+        }
+#ifdef CHUNKDEBUG
+        std::cerr << thread_id << "readChunk  size=" << size << std::endl;
+#endif
+        String l = size;
+        l.chop();
+        String t = l.before(";");
+        if (t.length() > 0) {
+            if (l.endsWith("; ieof")) {
+                ieof = true;
+            }
+            l = t;
+        }
+        chunk_to_read = l.hexToInteger();
+#ifdef CHUNKDEBUG
+        std::cerr << thread_id << "readChunk  chunk_to_read =" << chunk_to_read << std::endl;
+#endif
+    }
+
+    int clen = chunk_to_read;
+    if (clen > maxlen) {
+        clen = maxlen;
+    }
+    int rc = 0;
+#ifdef CHUNKDEBUG
+    std::cerr << thread_id << "readChunk  max_read =" << clen << std::endl;
+#endif
+
+    if(clen == 0) {
+        chunked_trailer = "";
+        char trailer[32000];
+        int len = 3;
+        while( len > 2) {
+            len = getLine(trailer, 31900, timeout);
+            if (len > 2) {
+                chunked_trailer += trailer;
+                chunked_trailer += "\n";
+            }
+        }
+        return 0;
+    }
+
+    if (clen > 0) {
+        rc = readFromSocketn(buffin, clen, 0, timeout);
+#ifdef CHUNKDEBUG
+        std::cerr << thread_id << "readChunk  read " << rc << std::endl;
+#endif
+        if (rc < 0) {
+            chunkError = true;
+            return -1;
+        }
+        chunk_to_read -= rc;
+    }
+    if (chunk_to_read > 0)    // there is more to read in this chunk - so do not check for trailing \r\n
+        return rc;
+    char ts[2];
+    int len = readFromSocketn(ts, 2, 0, timeout);
+    if (len == 2 && ts[0] == '\r' && ts[1] == '\n') {
+        return rc;
+    } else {
+        chunkError = true;
+#ifdef CHUNKDEBUG
+        std::cerr << thread_id << "readChunk - tail in error" << std::endl;
+#endif
+        return -1;
+    }
+}
+
+int Socket::loopChunk(int timeout)    // reads chunks and sends back until 0 len chunk or timeout
+{
+    char buff[32000];
+    int tot_size = 0;
+    int csize = 1;
+    while (csize > 0) {
+        csize = readChunk(buff,32000, timeout);
+        if (csize == 0)     // end chunk
+        {
+            if (!writeChunkTrailer(chunked_trailer))
+            {
+#ifdef CHUNKDEBUG
+                std::cerr << thread_id << "loopChunk - error in writing chunk trailer" << std::endl;
+#endif
+                return -1;
+
+            };
+#ifdef CHUNKDEBUG
+            std::cerr << thread_id << "loopChunk  tot_size=" << tot_size << std::endl;
+#endif
+            return tot_size;
+        }
+        if (!(csize > 0 && writeChunk(buff,csize,timeout))) {
+#ifdef CHUNKDEBUG
+            std::cerr << thread_id << "loopChunk - error" << std::endl;
+#endif
+            return -1;
+        }
+        tot_size += csize;
+    }
+}
+
+
+int Socket::drainChunk(int timeout)    // reads chunks until 0 len chunk or timeout
+{
+    char buff[32000];
+    int tot_size = 0;
+    int csize = 1;
+    while (csize > 0) {
+        csize = readChunk(buff,32000, timeout);
+        if (!(csize > -1 )) {
+#ifdef CHUNKDEBUG
+            std::cerr << thread_id << "drainChunk - error" << std::endl;
+#endif
+            return -1;
+        }
+        tot_size += csize;
+    }
+#ifdef CHUNKDEBUG
+    std::cerr << thread_id << "drainChunk  tot_size=" << tot_size << std::endl;
+#endif
+    return tot_size;
+}
+
+bool Socket::getIeof() {
+    return ieof;
+}
