@@ -15,7 +15,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 // INCLUDES
 
@@ -37,6 +37,7 @@
 
 extern bool is_daemonised;
 extern OptionContainer o;
+extern thread_local std::string thread_id;
 
 // DECLARATIONS
 
@@ -56,10 +57,11 @@ class portinstance : public AuthPlugin
         : AuthPlugin(definition)
     {
         is_connection_based = true;
+        client_ip_based = false;
     }
 
-    int identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string);
-    int determineGroup(std::string &user, int &fg);
+    int identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, std::string &string, bool &is_real_user);
+    int determineGroup(std::string &user, int &fg, ListContainer &uglc);
 
     int init(void *args);
     int quit();
@@ -104,7 +106,7 @@ int portinstance::init(void *args)
         return readIPMelangeList(fname.toCharArray());
     } else {
         if (!is_daemonised)
-            std::cerr << "No portgroups file defined in port auth plugin config" << std::endl;
+            std::cerr << thread_id << "No portgroups file defined in port auth plugin config" << std::endl;
         syslog(LOG_ERR, "No portgroups file defined in port auth plugin config");
         return -1;
     }
@@ -113,7 +115,7 @@ int portinstance::init(void *args)
 // Port-based filter group determination
 // never actually return NOUSER from this, because we don't actually look in the portgroupslist.
 // NOUSER stops ConnectionHandler from querying subsequent plugins.
-int portinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, /*int &fg,*/ std::string &string)
+int portinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, /*int &fg,*/ std::string &string, bool &is_real_user)
 {
     // we don't get usernames out of this plugin, just a filter group
     // for now, use the dest Port as the username
@@ -123,19 +125,21 @@ int portinstance::identify(Socket &peercon, Socket &proxycon, HTTPHeader &h, /*i
     return DGAUTH_OK;
 }
 
-int portinstance::determineGroup(std::string &user, int &fg)
+int portinstance::determineGroup(std::string &user, int &pfg, ListContainer &uglc)
 {
     // check ports
     String s = user;
+    int fg;
     fg = inList(s.toInteger());
     if (fg >= 0) {
+        pfg = fg;
 #ifdef DGDEBUG
-        std::cout << "Matched port " << user << " to port list" << std::endl;
+        std::cerr << thread_id << "Matched port " << user << " to port list" << std::endl;
 #endif
         return DGAUTH_OK;
     }
 #ifdef DGDEBUG
-    std::cout << "Matched port " << user << " to nothing" << std::endl;
+    std::cerr << thread_id << "Matched port " << user << " to nothing" << std::endl;
 #endif
     return DGAUTH_NOMATCH;
 }
@@ -178,7 +182,7 @@ int portinstance::readIPMelangeList(const char *filename)
     std::ifstream input(filename);
     if (!input) {
         if (!is_daemonised) {
-            std::cerr << "Error reading file (does it exist?): " << filename << std::endl;
+            std::cerr << thread_id << "Error reading file (does it exist?): " << filename << std::endl;
         }
         syslog(LOG_ERR, "%s%s", "Error reading file (does it exist?): ", filename);
         return -1;
@@ -207,18 +211,18 @@ int portinstance::readIPMelangeList(const char *filename)
             value = line.after("filter");
         } else {
             if (!is_daemonised)
-                std::cerr << "No filter group given; entry " << line << " in " << filename << std::endl;
+                std::cerr << thread_id << "No filter group given; entry " << line << " in " << filename << std::endl;
             syslog(LOG_ERR, "No filter group given; entry %s in %s", line.toCharArray(), filename);
             warn = true;
             continue;
         }
 #ifdef DGDEBUG
-        std::cout << "key: " << key << std::endl;
-        std::cout << "value: " << value.toInteger() << std::endl;
+        std::cerr << thread_id << "key: " << key << std::endl;
+        std::cerr << thread_id << "value: " << value.toInteger() << std::endl;
 #endif
         if ((value.toInteger() < 1) || (value.toInteger() > o.filter_groups)) {
             if (!is_daemonised)
-                std::cerr << "Filter group out of range; entry " << line << " in " << filename << std::endl;
+                std::cerr << thread_id << "Filter group out of range; entry " << line << " in " << filename << std::endl;
             syslog(LOG_ERR, "Filter group out of range; entry %s in %s", line.toCharArray(), filename);
             warn = true;
             continue;
@@ -233,22 +237,22 @@ int portinstance::readIPMelangeList(const char *filename)
         // hmmm. the key didn't match any of our regular expressions. output message & return a warning value.
         else {
             if (!is_daemonised)
-                std::cerr << "Entry " << line << " in " << filename << " was not recognised as an port " << std::endl;
+                std::cerr << thread_id << "Entry " << line << " in " << filename << " was not recognised as an port " << std::endl;
             syslog(LOG_ERR, "Entry %s in %s was not recognised as an port", line.toCharArray(), filename);
             warn = true;
         }
     }
     input.close();
 #ifdef DGDEBUG
-    std::cout << "starting sort" << std::endl;
+    std::cerr << thread_id << "starting sort" << std::endl;
 #endif
 //	std::sort(ipportlist.begin(), ipportlist.end());
 #ifdef DGDEBUG
-    std::cout << "sort complete" << std::endl;
-    std::cout << "port list dump:" << std::endl;
+    std::cerr << thread_id << "sort complete" << std::endl;
+    std::cerr << thread_id << "port list dump:" << std::endl;
     std::deque<portstruct>::iterator i = ipportlist.begin();
     while (i != ipportlist.end()) {
-        std::cout << "port: " << i->port << " Group: " << i->group << std::endl;
+        std::cerr << thread_id << "port: " << i->port << " Group: " << i->group << std::endl;
         i++;
     }
 #endif
