@@ -32,6 +32,7 @@
 
 extern OptionContainer o;
 extern bool is_daemonised;
+extern thread_local std::string thread_id;
 
 // DECLARATIONS
 
@@ -40,15 +41,15 @@ class icapinstance : public CSPlugin
 {
     public:
     icapinstance(ConfigVar &definition)
-        : CSPlugin(definition), usepreviews(false), previewforce(false), previewsize(0), supportsXIF(false), needsBody(false){};
+        : CSPlugin(definition), usepreviews(false), previewsize(0), supportsXIF(false), needsBody(false){};
 
-    int willScanRequest(const String &url, const char *user, int filtergroup, const char *ip, bool post,
+    int willScanRequest(const String &url, const char *user, FOptionContainer* &foc, const char *ip, bool post,
         bool reconstituted, bool exception, bool bypass);
 
-    int scanMemory(HTTPHeader *requestheader, HTTPHeader *docheader, const char *user, int filtergroup,
+    int scanMemory(HTTPHeader *requestheader, HTTPHeader *docheader, const char *user, FOptionContainer* &foc,
         const char *ip, const char *object, unsigned int objectsize, NaughtyFilter *checkme,
         const String *disposition, const String *mimetype);
-    int scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, const char *user, int filtergroup,
+    int scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, const char *user, FOptionContainer* &foc,
         const char *ip, const char *filename, NaughtyFilter *checkme,
         const String *disposition, const String *mimetype);
 
@@ -61,7 +62,6 @@ class icapinstance : public CSPlugin
     unsigned int icapport;
     // URL for the AV service
     String icapurl;
-    bool previewforce;
     // whether or not to send ICAP message previews, and the preview object size
     bool usepreviews;
     unsigned int previewsize;
@@ -88,13 +88,13 @@ CSPlugin *icapcreate(ConfigVar &definition)
 
 // don't scan POST data or reconstituted data - wouldn't work for multi-part posts
 // without faking request headers, as we are only passed a single part, not the whole request verbatim
-int icapinstance::willScanRequest(const String &url, const char *user, int filtergroup, const char *ip,
+int icapinstance::willScanRequest(const String &url, const char *user, FOptionContainer* &foc, const char *ip,
     bool post, bool reconstituted, bool exception, bool bypass)
 {
     if (post || reconstituted)
         return DGCS_NOSCAN;
     else {
-        return CSPlugin::willScanRequest(url, user, filtergroup, ip,
+        return CSPlugin::willScanRequest(url, user, foc, ip,
             post, reconstituted, exception, bypass);
     }
 }
@@ -107,15 +107,10 @@ int icapinstance::init(void *args)
         return DGCS_ERROR;
     }
 
-    if (cv["previewforce"] == "on"){
-    	previewforce = true;
-    }
-
-   icapurl = cv["icapurl"]; // format: icap://icapserver:1344/avscan
-
-   if (icapurl.length() < 3) {
+    icapurl = cv["icapurl"]; // format: icap://icapserver:1344/avscan
+    if (icapurl.length() < 3) {
         if (!is_daemonised)
-            std::cerr << "Error reading icapurl option." << std::endl;
+            std::cerr << thread_id << "Error reading icapurl option." << std::endl;
         syslog(LOG_ERR, "Error reading icapurl option.");
         return DGCS_ERROR;
         // it would be far better to do a test connection
@@ -132,14 +127,14 @@ int icapinstance::init(void *args)
     struct hostent *host;
     if ((host = gethostbyname(icaphost.toCharArray())) == 0) {
         if (!is_daemonised)
-            std::cerr << "Error resolving icap host address." << std::endl;
+            std::cerr << thread_id << "Error resolving icap host address." << std::endl;
         syslog(LOG_ERR, "Error resolving icap host address.");
         return DGCS_ERROR;
     }
     icapip = inet_ntoa(*(struct in_addr *)host->h_addr_list[0]);
 
 #ifdef DGDEBUG
-    std::cerr << "ICAP server address:" << icapip << std::endl;
+    std::cerr << thread_id << "ICAP server address:" << icapip << std::endl;
 #endif
 
     // try to connect to the ICAP server and perform an OPTIONS request
@@ -156,12 +151,12 @@ int icapinstance::init(void *args)
         icapsock.getLine(buff, 8192, o.content_scanner_timeout);
         line = buff;
 #ifdef DGDEBUG
-        std::cout << "ICAP/1.0 OPTIONS response:" << std::endl
+        std::cerr << thread_id << "ICAP/1.0 OPTIONS response:" << std::endl
                   << line << std::endl;
 #endif
         if (line.after(" ").before(" ") != "200") {
             if (!is_daemonised)
-                std::cerr << "ICAP response not 200 OK" << std::endl;
+                std::cerr << thread_id << "ICAP response not 200 OK" << std::endl;
             syslog(LOG_ERR, "ICAP response not 200 OK");
             return DGCS_WARNING;
             //throw std::runtime_error("Response not 200 OK");
@@ -169,7 +164,7 @@ int icapinstance::init(void *args)
         while (icapsock.getLine(buff, 8192, o.content_scanner_timeout) > 0) {
             line = buff;
 #ifdef DGDEBUG
-            std::cout << line << std::endl;
+            std::cerr << thread_id << line << std::endl;
 #endif
             if (line.startsWith("\r")) {
                 break;
@@ -189,21 +184,21 @@ int icapinstance::init(void *args)
         icapsock.close();
     } catch (std::exception &e) {
         if (!is_daemonised)
-            std::cerr << "ICAP server did not respond to OPTIONS request: " << e.what() << std::endl;
+            std::cerr << thread_id << "ICAP server did not respond to OPTIONS request: " << e.what() << std::endl;
         syslog(LOG_ERR, "ICAP server did not respond to OPTIONS request: %s", e.what());
         return DGCS_ERROR;
     }
 #ifdef DGDEBUG
     if (usepreviews)
-        std::cout << "Message previews enabled; size: " << previewsize << std::endl;
+        std::cerr << thread_id << "Message previews enabled; size: " << previewsize << std::endl;
     else
-        std::cout << "Message previews disabled" << std::endl;
+        std::cerr << thread_id << "Message previews disabled" << std::endl;
 #endif
     return DGCS_OK;
 }
 
 // send memory buffer to ICAP server for scanning
-int icapinstance::scanMemory(HTTPHeader *requestheader, HTTPHeader *docheader, const char *user, int filtergroup,
+int icapinstance::scanMemory(HTTPHeader *requestheader, HTTPHeader *docheader, const char *user, FOptionContainer* &foc,
     const char *ip, const char *object, unsigned int objectsize, NaughtyFilter *checkme,
     const String *disposition, const String *mimetype)
 {
@@ -216,12 +211,12 @@ int icapinstance::scanMemory(HTTPHeader *requestheader, HTTPHeader *docheader, c
         return DGCS_SCANERROR;
     }
 #ifdef DGDEBUG
-    std::cerr << "About to send memory data to icap" << std::endl;
-    if (usepreviews && ((objectsize > previewsize) || previewforce))
-        std::cerr << "Sending preview first" << std::endl;
+    std::cerr << thread_id << "About to send memory data to icap" << std::endl;
+    if (usepreviews && (objectsize > previewsize))
+        std::cerr << thread_id << "Sending preview first" << std::endl;
 #endif
     unsigned int sent = 0;
-    if (usepreviews && ((objectsize > previewsize) || previewforce)) {
+    if (usepreviews && (objectsize > previewsize)) {
         try {
             if (!icapsock.writeToSocket(object, previewsize, 0, o.content_scanner_timeout)) {
                 throw std::runtime_error("standard error");
@@ -242,7 +237,7 @@ int icapinstance::scanMemory(HTTPHeader *requestheader, HTTPHeader *docheader, c
             icapsock.writeString(objectsizehex);
         } catch (std::exception &e) {
 #ifdef DGDEBUG
-            std::cerr << "Exception sending message preview to ICAP: " << e.what() << std::endl;
+            std::cerr << thread_id << "Exception sending message preview to ICAP: " << e.what() << std::endl;
 #endif
             // this *might* just be an early response & closed connection
             if (icapsock.checkForInput()) {
@@ -257,17 +252,18 @@ int icapinstance::scanMemory(HTTPHeader *requestheader, HTTPHeader *docheader, c
         }
     }
     try {
-        icapsock.writeToSockete(object + sent, objectsize - sent, 0, o.content_scanner_timeout);
+        if(icapsock.writeToSocket(object + sent, objectsize - sent, 0, o.content_scanner_timeout)) {
 #ifdef DGDEBUG
-        std::cout << "total sent to icap: " << objectsize << std::endl;
+            std::cerr << thread_id << "total sent to icap: " << objectsize << std::endl;
 #endif
-        icapsock.writeString("\r\n0\r\n\r\n"); // end marker
+            icapsock.writeString("\r\n0\r\n\r\n"); // end marker
 #ifdef DGDEBUG
-        std::cout << "memory was sent to icap" << std::endl;
+            std::cerr << thread_id << "memory was sent to icap" << std::endl;
 #endif
+        }
     } catch (std::exception &e) {
 #ifdef DGDEBUG
-        std::cerr << "Exception sending memory file to ICAP: " << e.what() << std::endl;
+        std::cerr << thread_id << "Exception sending memory file to ICAP: " << e.what() << std::endl;
 #endif
         // this *might* just be an early response & closed connection
         if (icapsock.checkForInput()) {
@@ -286,14 +282,14 @@ int icapinstance::scanMemory(HTTPHeader *requestheader, HTTPHeader *docheader, c
 
 // send file contents for scanning
 int icapinstance::scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, const char *user,
-    int filtergroup, const char *ip, const char *filename, NaughtyFilter *checkme,
+    FOptionContainer* &foc, const char *ip, const char *filename, NaughtyFilter *checkme,
     const String *disposition, const String *mimetype)
 {
     lastmessage = lastvirusname = "";
     int filefd = open(filename, O_RDONLY);
     if (filefd < 0) {
 #ifdef DGDEBUG
-        std::cerr << "Error opening file (" << filename << "): " << strerror(errno) << std::endl;
+        std::cerr << thread_id << "Error opening file (" << filename << "): " << strerror(errno) << std::endl;
 #endif
         lastmessage = "Error opening file to send to ICAP";
         syslog(LOG_ERR, "Error opening file to send to ICAP: %s", strerror(errno));
@@ -316,9 +312,9 @@ int icapinstance::scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, con
     int objectsize = 0;
 
 #ifdef DGDEBUG
-    std::cerr << "About to send file data to icap" << std::endl;
+    std::cerr << thread_id << "About to send file data to icap" << std::endl;
     if (usepreviews && (filesize > previewsize))
-        std::cerr << "Sending preview first" << std::endl;
+        std::cerr << thread_id << "Sending preview first" << std::endl;
 #endif
     if (usepreviews && (filesize > previewsize)) {
         try {
@@ -358,7 +354,7 @@ int icapinstance::scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, con
             icapsock.writeString(objectsizehex);
         } catch (std::exception &e) {
 #ifdef DGDEBUG
-            std::cerr << "Exception sending message preview to ICAP: " << e.what() << std::endl;
+            std::cerr << thread_id << "Exception sending message preview to ICAP: " << e.what() << std::endl;
 #endif
             icapsock.close();
             lastmessage = "Exception sending message preview to ICAP";
@@ -382,35 +378,37 @@ int icapinstance::scanFile(HTTPHeader *requestheader, HTTPHeader *docheader, con
         while (sent < filesize) {
             int rc = readEINTR(filefd, data, 256 * 1024);
 #ifdef DGDEBUG
-            std::cout << "reading icap file rc: " << rc << std::endl;
+            std::cerr << thread_id << "reading icap file rc: " << rc << std::endl;
 #endif
             if (rc < 0) {
 #ifdef DGDEBUG
-                std::cout << "error reading icap file so throwing exception" << std::endl;
+                std::cerr << thread_id << "error reading icap file so throwing exception" << std::endl;
 #endif
                 throw std::runtime_error("could not read from file");
             }
             if (rc == 0) {
 #ifdef DGDEBUG
-                std::cout << "got zero bytes reading icap file" << std::endl;
+                std::cerr << thread_id << "got zero bytes reading icap file" << std::endl;
 #endif
                 break; // should never happen
             }
             memcpy(object + objectsize, data, (rc > (100 - objectsize)) ? (100 - objectsize) : rc);
             objectsize += (rc > (100 - objectsize)) ? (100 - objectsize) : rc;
-            icapsock.writeToSockete(data, rc, 0, o.content_scanner_timeout);
+            if (!icapsock.writeToSocket(data, rc, 0, o.content_scanner_timeout)) {
+                throw std::runtime_error("Unable to write to socket");
+            };
             sent += rc;
         }
 #ifdef DGDEBUG
-        std::cout << "total sent to icap: " << sent << std::endl;
+        std::cerr << thread_id << "total sent to icap: " << sent << std::endl;
 #endif
         icapsock.writeString("\r\n0\r\n\r\n"); // end marker
 #ifdef DGDEBUG
-        std::cout << "file was sent to icap" << std::endl;
+        std::cerr << thread_id << "file was sent to icap" << std::endl;
 #endif
     } catch (std::exception &e) {
 #ifdef DGDEBUG
-        std::cerr << "Exception sending file to ICAP: " << e.what() << std::endl;
+        std::cerr << thread_id << "Exception sending file to ICAP: " << e.what() << std::endl;
 #endif
         lastmessage = "Exception sending file to ICAP";
         syslog(LOG_ERR, "Exception sending file to ICAP: %s", e.what());
@@ -435,7 +433,7 @@ bool icapinstance::doHeaders(Socket &icapsock, HTTPHeader *reqheader, HTTPHeader
     int rc = icapsock.connect(icapip.toCharArray(), icapport);
     if (rc) {
 #ifdef DGDEBUG
-        std::cerr << "Error connecting to ICAP server" << std::endl;
+        std::cerr << thread_id << "Error connecting to ICAP server" << std::endl;
 #endif
         lastmessage = "Error connecting to ICAP server";
         syslog(LOG_ERR, "Error connecting to ICAP server");
@@ -448,7 +446,7 @@ bool icapinstance::doHeaders(Socket &icapsock, HTTPHeader *reqheader, HTTPHeader
     // leakage over the network.
     String encapsulatedheader("GET " + reqheader->getUrl() + " HTTP/1.0\r\n\r\n");
     // body chunk size in hex - either full body, or just preview
-    if (usepreviews && ((objectsize > previewsize) || previewforce)) {
+    if (usepreviews && (objectsize > previewsize)) {
         snprintf(objectsizehex, sizeof(objectsizehex), "%x\r\n", previewsize);
     } else {
         snprintf(objectsizehex, sizeof(objectsizehex), "%x\r\n", objectsize);
@@ -463,13 +461,13 @@ bool icapinstance::doHeaders(Socket &icapsock, HTTPHeader *reqheader, HTTPHeader
     String httpresponseheader("HTTP/1.0 200 OK\r\n\r\n");
     // ICAP header itself
     String icapheader("RESPMOD " + icapurl + " ICAP/1.0\r\nHost: " + icaphost + "\r\nAllow: 204\r\nEncapsulated: req-hdr=0, res-hdr=" + String(encapsulatedheader.length()) + ", res-body=" + String(httpresponseheader.length() + encapsulatedheader.length()));
-    if (usepreviews && ((objectsize > previewsize) || previewforce)) {
+    if (usepreviews && (objectsize > previewsize)) {
         icapheader += "\r\nPreview: " + String(previewsize);
-   }
+    }
     icapheader += "\r\n\r\n";
 
 #ifdef DGDEBUG
-    std::cerr << "About to send icapheader:\n" << icapheader << encapsulatedheader << httpresponseheader << objectsizehex << std::endl;
+    std::cerr << thread_id << "About to send icapheader:\n" << icapheader << encapsulatedheader << httpresponseheader << objectsizehex << std::endl;
 #endif
     try {
         icapsock.writeString(icapheader.toCharArray());
@@ -478,7 +476,7 @@ bool icapinstance::doHeaders(Socket &icapsock, HTTPHeader *reqheader, HTTPHeader
         icapsock.writeString(objectsizehex);
     } catch (std::exception &e) {
 #ifdef DGDEBUG
-        std::cerr << "Exception sending headers to ICAP: " << e.what() << std::endl;
+        std::cerr << thread_id << "Exception sending headers to ICAP: " << e.what() << std::endl;
 #endif
         lastmessage = "Exception sending headers to ICAP";
         syslog(LOG_ERR, "Exception sending headers to ICAP: %s", e.what());
@@ -498,7 +496,7 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
             return ICAP_NODATA;
         line = data;
 #ifdef DGDEBUG
-        std::cout << "reply from icap: " << line << std::endl;
+        std::cerr << thread_id << "reply from icap: " << line << std::endl;
 #endif
         // reply is of the format:
         // ICAP/1.0 204 No Content Necessary (etc)
@@ -507,13 +505,13 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
 
         if (returncode == "204") {
 #ifdef DGDEBUG
-            std::cerr << "ICAP says clean!" << std::endl;
+            std::cerr << thread_id << "ICAP says clean!" << std::endl;
 #endif
             delete[] data;
             return DGCS_CLEAN;
         } else if (returncode == "100") {
 #ifdef DGDEBUG
-            std::cerr << "ICAP says continue!" << std::endl;
+            std::cerr << thread_id << "ICAP says continue!" << std::endl;
 #endif
             // discard rest of headers (usually just a blank line)
             // this is so we are in the right place in the data stream to
@@ -528,7 +526,7 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
             return ICAP_CONTINUE;
         } else if (returncode == "200") {
 #ifdef DGDEBUG
-            std::cerr << "ICAP says maybe not clean!" << std::endl;
+            std::cerr << thread_id << "ICAP says maybe not clean!" << std::endl;
 #endif
             while (icapsock.getLine(data, 8192, o.content_scanner_timeout) > 0) {
                 if (data[0] == 13) // end marker
@@ -537,7 +535,7 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
                 // Symantec's engine gives us the virus name in the ICAP headers
                 if (supportsXIF && line.startsWith("X-Infection-Found")) {
 #ifdef DGDEBUG
-                    std::cout << "ICAP says infected! (X-Infection-Found)" << std::endl;
+                    std::cerr << thread_id << "ICAP says infected! (X-Infection-Found)" << std::endl;
 #endif
                     lastvirusname = line.after("Threat=").before(";");
                     delete[] data;
@@ -555,14 +553,14 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
                 icapsock.getLine(data, 8192, o.content_scanner_timeout);
                 line = data;
 #ifdef DGDEBUG
-                std::cout << "Comparing original return code to modified:" << std::endl
+                std::cerr << thread_id << "Comparing original return code to modified:" << std::endl
                           << docheader->header.front() << std::endl
                           << line << std::endl;
 #endif
                 int respmodReturnCode = line.after(" ").before(" ").toInteger();
                 if (respmodReturnCode != docheader->returnCode()) {
 #ifdef DGDEBUG
-                    std::cerr << "ICAP says infected! (returned header comparison)" << std::endl;
+                    std::cerr << thread_id << "ICAP says infected! (returned header comparison)" << std::endl;
 #endif
                     delete[] data;
                     lastvirusname = "Unknown";
@@ -578,7 +576,7 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
                 }
 // grab body chunk size
 #ifdef DGDEBUG
-                std::cout << "Comparing original body data to modified" << std::endl;
+                std::cerr << thread_id << "Comparing original body data to modified" << std::endl;
 #endif
                 icapsock.getLine(data, 8192, o.content_scanner_timeout);
                 line = data;
@@ -591,13 +589,13 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
                 icapsock.readFromSocket(data, chunksize, 0, o.content_scanner_timeout);
                 if (memcmp(data, object, chunksize) == 0) {
 #ifdef DGDEBUG
-                    std::cerr << "ICAP says clean!" << std::endl;
+                    std::cerr << thread_id << "ICAP says clean!" << std::endl;
 #endif
                     delete[] data;
                     return DGCS_CLEAN;
                 } else {
 #ifdef DGDEBUG
-                    std::cerr << "ICAP says infected! (body byte comparison)" << std::endl;
+                    std::cerr << thread_id << "ICAP says infected! (body byte comparison)" << std::endl;
 #endif
                     delete[] data;
                     lastvirusname = "Unknown";
@@ -609,7 +607,7 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
 // even if we don't find an X-Infection-Found header,
 // the file is still infected!
 #ifdef DGDEBUG
-            std::cerr << "ICAP says infected! (no further tests)" << std::endl;
+            std::cerr << thread_id << "ICAP says infected! (no further tests)" << std::endl;
 #endif
             delete[] data;
             lastvirusname = "Unknown";
@@ -618,7 +616,7 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
             return DGCS_INFECTED;
         } else if (returncode == "404") {
 #ifdef DGDEBUG
-            std::cerr << "ICAP says no such service!" << std::endl;
+            std::cerr << thread_id << "ICAP says no such service!" << std::endl;
 #endif
             lastmessage = "ICAP reports no such service";
             syslog(LOG_ERR, "ICAP reports no such service; check your server URL");
@@ -626,7 +624,7 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
             return DGCS_SCANERROR;
         } else {
 #ifdef DGDEBUG
-            std::cerr << "ICAP returned unrecognised response code: " << returncode << std::endl;
+            std::cerr << thread_id << "ICAP returned unrecognised response code: " << returncode << std::endl;
 #endif
             lastmessage = "ICAP returned unrecognised response code.";
             syslog(LOG_ERR, "ICAP returned unrecognised response code: %s", returncode.toCharArray());
@@ -636,7 +634,7 @@ int icapinstance::doScan(Socket &icapsock, HTTPHeader *docheader, const char *ob
         delete[] data;
     } catch (std::exception &e) {
 #ifdef DGDEBUG
-        std::cerr << "Exception getting reply from ICAP: " << e.what() << std::endl;
+        std::cerr << thread_id << "Exception getting reply from ICAP: " << e.what() << std::endl;
 #endif
         lastmessage = "Exception getting reply from ICAP.";
         syslog(LOG_ERR, "Exception getting reply from ICAP: %s", e.what());
