@@ -226,7 +226,8 @@ int trickledm::in(DataBuffer *d, Socket *sock, Socket *peersock, class HTTPHeade
             delete[] block;
             block = new char[newsize];
             try {
-                sock->checkForInput(d->timeout);
+                if(!sock->bcheckForInput(d->timeout))
+                    break;
             } catch (std::exception &e) {
                 break;
             }
@@ -235,7 +236,7 @@ int trickledm::in(DataBuffer *d, Socket *sock, Socket *peersock, class HTTPHeade
             // grab a block of input, doubled each time
 
             if (rc <= 0) {
-                break; // an error occurred so end the while()
+                break; // an error occured so end the while()
                 // or none received so pipe is closed
             } else {
                 bytesremaining -= rc;
@@ -252,7 +253,8 @@ int trickledm::in(DataBuffer *d, Socket *sock, Socket *peersock, class HTTPHeade
             }
         } else {
             try {
-                sock->checkForInput(d->timeout);
+                if(!sock->bcheckForInput(d->timeout))
+                    break;
             } catch (std::exception &e) {
                 break;
             }
